@@ -1,8 +1,6 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from __future__ import with_statement
 
 import tests
 
@@ -17,6 +15,7 @@ from zim.newfs import LocalFolder, File
 from zim.newfs.mock import os_native_path
 
 from zim.notebook import Path
+from zim.notebook.index import Index, DB_VERSION
 from zim.notebook.index.files import FilesIndexer, TestFilesDBTable, FilesIndexChecker
 from zim.notebook.index.pages import PagesIndexer, TestPagesDBTable
 from zim.notebook.index.links import LinksIndexer
@@ -27,6 +26,68 @@ def is_dir(path):
 	return path.endswith('/') or path.endswith('\\')
 
 
+@tests.slowTest
+class TestIndexInitialization(tests.TestCase):
+
+	def setUp(self):
+		self.folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		self.folder.touch() # Must exist for sane notebook
+		self.layout = FilesLayout(self.folder)
+
+	def testWithoutFileAndWithValidFile(self):
+		# Two tests combined because first needed as init for the second
+		file = self.folder.file('index.db')
+		self.assertFalse(file.exists())
+		index = Index(file.path, self.layout)
+		self.assertTrue(file.exists())
+		self.assertEqual(index.get_property('db_version'), DB_VERSION)
+
+		index._db.close()
+		del(index)
+
+		index = Index(file.path, self.layout)
+		self.assertTrue(file.exists())
+		self.assertEqual(index.get_property('db_version'), DB_VERSION)
+
+	def testWithValidDBFile(self):
+		# E.g. old index, not conforming our table layout
+		file = self.folder.file('index.db')
+		self.assertFalse(file.exists())
+
+		db = sqlite3.Connection(file.path)
+		db.execute('CREATE TABLE zim_index (key TEXT);')
+		db.close()
+
+		self.assertTrue(file.exists())
+		index = Index(file.path, self.layout)
+		self.assertTrue(file.exists())
+		self.assertEqual(index.get_property('db_version'), DB_VERSION)
+
+	def testWithBrokenFile(self):
+		file = self.folder.file('index.db')
+		file.write('this is not a database file...\n')
+
+		self.assertTrue(file.exists())
+		with tests.LoggingFilter('zim.notebook.index', 'Overwriting'):
+			with tests.LoggingFilter('zim.notebook.index', 'Could not access'):
+				index = Index(file.path, self.layout)
+		self.assertTrue(file.exists())
+		self.assertEqual(index.get_property('db_version'), DB_VERSION)
+
+	def testWithLockedFile(self):
+		file = self.folder.file('index.db')
+		file.write('this is not a database file...\n')
+		os.chmod(file.path, 0o000) # make read-only
+		self.addCleanup(lambda: os.chmod(file.path, 0o700))
+
+		self.assertTrue(file.exists())
+		with tests.LoggingFilter('zim.notebook.index', 'Overwriting'):
+			with tests.LoggingFilter('zim.notebook.index', 'Could not access'):
+				index = Index(file.path, self.layout)
+		self.assertTrue(file.exists())
+		self.assertEqual(index.get_property('db_version'), DB_VERSION)
+
+
 class TestFilesIndexer(tests.TestCase, TestFilesDBTable):
 
 	FILES = tuple(map(os_native_path, (
@@ -74,7 +135,7 @@ class TestFilesIndexer(tests.TestCase, TestFilesDBTable):
 		indexer = FilesIndexer(db, self.root)
 
 		def cb_filter_func(name, o, a):
-			#~ print '>>', name
+			#~ print('>>', name)
 			if name in ('start-update', 'finish-update'):
 				self.assertFalse(a)
 				return ()
@@ -321,13 +382,32 @@ class TestPagesIndexer(TestPagesDBTable, tests.TestCase):
 
 		self.assertPagesDBEquals(db, [])
 		self.assertEqual(signals['page-row-inserted'], [])
-		self.assertEqual(set(signals['page-row-changed']), set(['foo']))
+		self.assertEqual(set(signals['page-row-changed']), {'foo'})
 						 # "foo" has source that is deleted before children
 		self.assertEqual(set(signals['page-row-deleted']), set(self.PAGES))
 		self.assertEqual(signals['page-changed'], ['foo'])
 						 # "foo" has source that is deleted before children
 
 
+class TestPageNameConflict(tests.TestCase):
+
+	def runTest(self):
+		folder = self.setUpFolder()
+		layout = FilesLayout(folder)
+		db = sqlite3.connect(':memory:')
+		db.row_factory = sqlite3.Row
+
+		file_indexer = tests.MockObject()
+
+		indexer = PagesIndexer(db, layout, file_indexer)
+
+		id1 = indexer.insert_page(Path('Test'), None)
+		with tests.LoggingFilter('zim.notebook.index', 'Error while inserting page'):
+			id2 = indexer.insert_page(Path('Test'), None)
+
+		self.assertEqual(id1, id2)
+
+
 from zim.utils import natural_sort_key
 from zim.notebook.index.pages import PagesViewInternal
 from zim.notebook.page import HRef
@@ -430,7 +510,7 @@ class TestTagsIndexer(tests.TestCase):
 
 		for i, name, content in self.PAGES:
 			row = {'id': i, 'name': name}
-			indexer.on_page_row_deleted(None, row)
+			indexer.on_page_row_delete(None, row)
 		indexer.update()
 
 		self.assertTags(db, [], [])
