@@ -1,17 +1,18 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import tests
 
-from zim.plugins import PluginManager
+from zim.plugins import find_extension, PluginManager
+from zim.plugins.inlinecalculator import InlineCalculatorPageViewExtension
+
+from tests.pageview import setUpPageView
 
 
 @tests.slowTest
-class TestPrintToBrowser(tests.TestCase):
+class TestInlineCalculator(tests.TestCase):
 
 	def runTest(self):
-		'Test InlineCalculator plugin'
 		pluginklass = PluginManager.get_plugin_class('inlinecalculator')
 		plugin = pluginklass()
 
@@ -19,7 +20,7 @@ class TestPrintToBrowser(tests.TestCase):
 			('3 + 4 =', '3 + 4 = 7'),
 			('3 + 4 = 1', '3 + 4 = 7'),
 			('3 + 4 = 1 ', '3 + 4 = 7 '),
-			('10 / 3 =', '10 / 3 = 3.33333333333'), # divide integers to float !
+			('10 / 3 =', '10 / 3 = 3.3333333333333335'), # divide integers to float !
 			('milage: 3 + 4 =', 'milage: 3 + 4 = 7'),
 			('3 + 4 = 7 + 0.5 =  ', '3 + 4 = 7 + 0.5 = 7.5'),
 			('''\
@@ -63,8 +64,8 @@ angle( exp( j*pi ) ) == pi
 # log(-1)**2 == -1*pow(pi,2)
 round( degrees(phase( e**(2j)))) == 115
 # sum( [ round(42 * exp(j*2*x*pi/4)) for x in range(4)] ) == 0
-oct(8) == '010'
-0x42-042-42 == -10
+oct(8) == '0o10'
+0x42-0o42-42 == -10
 # 1k == 1024
 # 1m == 2**20
 # 1g == 2**30
@@ -73,9 +74,40 @@ oct(8) == '010'
 '''.splitlines():
 			if test.startswith('#'):
 				continue
-			#~ print 'TESTING:', test
+			# print('TESTING:', test)
 			self.assertTrue(plugin.safe_eval(test))
 
 		self.assertRaises(Exception, plugin.process_text, 'open("/etc/passwd")') # global
 		self.assertRaises(Exception, plugin.process_text, 'self') # local
 
+
+class TestInlineCalculatorExtension(tests.TestCase):
+
+	def runTest(self):
+		plugin = PluginManager.load_plugin('inlinecalculator')
+		notebook = self.setUpNotebook()
+		pageview = setUpPageView(notebook)
+
+		extension = find_extension(pageview, InlineCalculatorPageViewExtension)
+		buffer = pageview.textview.get_buffer()
+		def get_text():
+			start, end = buffer.get_bounds()
+			return start.get_text(end)
+
+		# Simple case
+		buffer.set_text('1 + 1 =\n')
+		buffer.place_cursor(buffer.get_iter_at_offset(7))
+		extension.eval_math()
+		self.assertEqual(get_text(), '1 + 1 = 2\n')
+
+		# Looks back to previous line
+		buffer.set_text('1 + 1 =\n\n')
+		buffer.place_cursor(buffer.get_iter_at_offset(8))
+		extension.eval_math()
+		self.assertEqual(get_text(), '1 + 1 = 2\n\n')
+
+		# Multi-line example
+		buffer.set_text('1\n2\n3\n--- +\n')
+		buffer.place_cursor(buffer.get_iter_at_offset(6))
+		extension.eval_math()
+		self.assertEqual(get_text(), '1\n2\n3\n--- +\n6\n')
