@@ -1,10 +1,9 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2012-2016 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the base zim module.'''
 
-from __future__ import with_statement
+
 
 
 import tests
@@ -12,13 +11,12 @@ import tests
 from tests.config import EnvironmentConfigContext, ConfigManager
 
 import sys
-import cStringIO as StringIO
+import io as StringIO
 import threading
 import time
 
 
 from zim.fs import Dir, File, FS
-from zim.environ import environ
 
 from zim.main import *
 
@@ -41,7 +39,7 @@ class capture_stdout:
 class TestParseCommand(tests.TestCase):
 
 	def runTest(self):
-		for command, klass in zim.main.commands.items():
+		for command, klass in list(zim.main.commands.items()):
 			obj = zim.main.build_command(['--%s' % command])
 			self.assertIsInstance(obj, klass)
 
@@ -103,11 +101,11 @@ class TestNotebookCommand(tests.TestCase):
 class TestGui(tests.TestCase):
 
 	def setUp(self):
-		config = ConfigManager() # XXX should be passed in
-		file = config.get_config_file('notebooks.list')
+		file = ConfigManager.get_config_file('notebooks.list')
 		file.remove()
 
 	def runTest(self):
+		from zim.gui.mainwindow import MainWindow
 
 		## Without argument should prompt
 		def testAddNotebookDialog(dialog):
@@ -117,20 +115,24 @@ class TestGui(tests.TestCase):
 
 		cmd = GuiCommand('gui')
 		with tests.DialogContext(testAddNotebookDialog):
-			cmd.run() # Exist without running due to no notebook given in dialog
+			cmd.run() # Exits without running due to no notebook given in dialog
 
 		### Try again with argument
 		dir = self.create_tmp_dir()
 		cmd = GuiCommand('gui')
 		cmd.parse_options(dir)
-		with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
-			window = cmd.run()
-		self.addCleanup(window.destroy)
+		with tests.WindowContext(MainWindow):
+			with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
+				window = cmd.run()
+				self.addCleanup(window.destroy)
 
 		self.assertEqual(window.__class__.__name__, 'MainWindow')
-		self.assertEqual(window.ui.notebook.uri, Dir(dir).uri) # XXX
+		self.assertEqual(window.notebook.uri, Dir(dir).uri) # XXX
+		self.assertGreaterEqual(len(ConfigManager.preferences['General']['plugins']), 3)
+		self.assertGreaterEqual(len(window.pageview.__zim_extension_objects__), 3)
 
-		window2 = cmd.run()
+		with tests.WindowContext(MainWindow):
+			window2 = cmd.run()
 		self.assertIs(window2, window)
 			# Ensure repeated calling gives unique window
 
@@ -142,10 +144,14 @@ class TestGui(tests.TestCase):
 class TestManual(tests.TestCase):
 
 	def runTest(self):
+		from zim.gui.mainwindow import MainWindow
+
 		cmd = ManualCommand('manual')
-		with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
-			window = cmd.run()
-		self.addCleanup(window.destroy)
+		with tests.WindowContext(MainWindow):
+			with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
+				window = cmd.run()
+				self.addCleanup(window.destroy)
+
 		self.assertEqual(window.__class__.__name__, 'MainWindow')
 
 
@@ -153,20 +159,27 @@ class TestManual(tests.TestCase):
 class TestServer(tests.TestCase):
 
 	def runTest(self):
-		from urllib import urlopen
+		from urllib.request import urlopen
+		from urllib.error import URLError
 
 		dir = self.create_tmp_dir()
 		cmd = ServerCommand('server')
 		cmd.parse_options(dir)
 		t = threading.Thread(target=cmd.run)
 		t.start()
-		try:
-			time.sleep(3) # give time to startup
-			re = urlopen('http://localhost:8080')
-			self.assertEqual(re.getcode(), 200)
-		finally:
-			cmd.server.shutdown()
-			t.join()
+
+		for i in range(30):
+			try:
+				re = urlopen('http://localhost:8080')
+				self.assertEqual(re.getcode(), 200)
+				break
+			except URLError:
+				time.sleep(1) # give more time to startup server
+		else:
+			assert False, 'Failed to start server within 10 seconds'
+
+		cmd.server.shutdown()
+		t.join()
 
 
 class TestServerGui(tests.TestCase):
@@ -255,17 +268,17 @@ class TestZimApplication(tests.TestCase):
 				self.hasrun = False
 
 			def _quit(self, *a):
-				import gobject
-				import gtk
-				gtk.main_quit()
+				from gi.repository import GObject
+				from gi.repository import Gtk
+				Gtk.main_quit()
 				return False # stop timer
 
 			def run(self):
-				import gobject
-				import gtk
+				from gi.repository import GObject
+				from gi.repository import Gtk
 				self.hasrun = True
-				gobject.timeout_add(500, self._quit)
-				return gtk.Window()
+				GObject.timeout_add(500, self._quit)
+				return Gtk.Window()
 
 		cmd = MockCmd()
 		self.assertFalse(cmd.hasrun)
@@ -300,7 +313,7 @@ class TestZimApplication(tests.TestCase):
 		app.add_window(w1)
 		app.add_window(w2)
 
-		self.assertEqual(set(app.toplevels), set([w1, w2]))
-		self.assertEqual(app.notebooks, set([n1, n2]))
+		self.assertEqual(set(app.toplevels), {w1, w2})
+		self.assertEqual(app.notebooks, {n1, n2})
 		self.assertEqual(app.get_mainwindow(n1, _class=MockWindow), w1)
 		self.assertEqual(app.get_mainwindow(MockNotebook('foo'), _class=MockWindow), w1)
