@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2012-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -47,30 +46,30 @@ class TestOrderedDict(tests.TestCase):
 
 		self.assertIsInstance(repr(mydict), str)
 
-		self.assertEqual(mydict.items(), items)
+		self.assertEqual(list(mydict.items()), items)
 		self.assertEqual(list(mydict), [i[0] for i in items])
-		self.assertEqual(mydict.keys(), [i[0] for i in items])
+		self.assertEqual(list(mydict.keys()), [i[0] for i in items])
 
 		mydict['bar'] = 'X'
 		mydict.setdefault('foo', 'dus')
 		items = [('foo', 1), ('bar', 'X'), ('baz', 3)]
-		self.assertEqual(mydict.items(), items)
+		self.assertEqual(list(mydict.items()), items)
 		self.assertEqual(list(mydict), [i[0] for i in items])
-		self.assertEqual(mydict.keys(), [i[0] for i in items])
+		self.assertEqual(list(mydict.keys()), [i[0] for i in items])
 
 		del mydict['bar']
 		mydict['bar'] = 'Y'
 		items = [('foo', 1), ('baz', 3), ('bar', 'Y')]
-		self.assertEqual(mydict.items(), items)
+		self.assertEqual(list(mydict.items()), items)
 		self.assertEqual(list(mydict), [i[0] for i in items])
-		self.assertEqual(mydict.keys(), [i[0] for i in items])
+		self.assertEqual(list(mydict.keys()), [i[0] for i in items])
 
 		mydict.pop('foo')
 		mydict.setdefault('foo', 'dus')
 		items = [('baz', 3), ('bar', 'Y'), ('foo', 'dus')]
-		self.assertEqual(mydict.items(), items)
+		self.assertEqual(list(mydict.items()), items)
 		self.assertEqual(list(mydict), [i[0] for i in items])
-		self.assertEqual(mydict.keys(), [i[0] for i in items])
+		self.assertEqual(list(mydict.keys()), [i[0] for i in items])
 
 
 class TestMovingWindowIterBuffer(tests.TestCase):
@@ -88,70 +87,11 @@ class TestMovingWindowIterBuffer(tests.TestCase):
 			if i == 0:
 				self.assertEqual(t, (None, mylist[0], mylist[1]))
 				self.assertFalse(myiter.last)
-			elif i == n-1:
+			elif i == n - 1:
 				self.assertEqual(t, (mylist[-2], mylist[-1], None))
 				self.assertTrue(myiter.last)
 			else:
-				self.assertEqual(t, (mylist[i-1], mylist[i], mylist[i+1]))
+				self.assertEqual(t, (mylist[i - 1], mylist[i], mylist[i + 1]))
 				self.assertFalse(myiter.last)
 
 		self.assertEqual(seen, mylist)
-
-
-import threading
-
-class TestFunctionThread(tests.TestCase):
-
-	def runTest(self):
-
-		def foo(*args):
-			return 'FOO: ' + ', '.join(args)
-
-		# Function OK, no lock
-		func = FunctionThread(foo, ('a', 'b', 'c'))
-		self.assertFalse(func.done)
-		func.start()
-
-		func.join()
-		self.assertTrue(func.done)
-		self.assertFalse(func.error)
-		self.assertEqual(func.result, 'FOO: a, b, c')
-
-		# Function OK, with lock
-		lock = threading.Lock()
-
-		func = FunctionThread(foo, ('a', 'b', 'c'), lock=lock)
-		self.assertFalse(func.done)
-		func.start()
-
-		lock.acquire()
-		self.assertTrue(func.done)
-		self.assertFalse(func.error)
-		self.assertEqual(func.result, 'FOO: a, b, c')
-
-		###
-
-		def error(*args):
-			raise AssertionError, 'FOO'
-
-		# Function raises, no lock
-		func = FunctionThread(error, ('a', 'b', 'c'))
-		self.assertFalse(func.done)
-		func.start()
-
-		func.join()
-		self.assertTrue(func.done)
-		self.assertTrue(func.error)
-		self.assertEqual(func.exc_info[0], AssertionError)
-
-		# Function raises, with lock
-		#~ lock = threading.Lock()
-
-		#~ func = FunctionThread(error, ('a', 'b', 'c'))
-		#~ self.assertFalse(func.done)
-		#~ func.start()
-
-		#~ lock.acquire()
-		#~ self.assertTrue(func.done)
-		#~ self.assertTrue(func.error)
-		#~ self.assertEqual(func.exc_info[0], AssertionError)
