@@ -1,10 +1,10 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 import os
 import subprocess
 
-from BaseHTTPServer import HTTPServer
-from SimpleHTTPServer import SimpleHTTPRequestHandler
+from http.server import HTTPServer
+from http.server import SimpleHTTPRequestHandler
 
 subprocess.call('./tools/build_website.sh')
 
@@ -12,8 +12,7 @@ os.chdir('./html')
 
 try:
 	server = HTTPServer(('', 8080), SimpleHTTPRequestHandler)
-	print 'server started at http://localhost:8080'
+	print('server started at http://localhost:8080')
 	server.serve_forever()
 except KeyboardInterrupt:
 	server.socket.close()
-
