@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -18,10 +17,10 @@ Both derive from L{ControlledDict} which defines the C{changed} signal
 which can be used to track changes in the configuration.
 
 Typically these classes are not instantiated directly, but by the
-L{ConfigManager} defined in Lzim.config.manager}.
+L{ConfigManager} defined in L{zim.config.manager}.
 '''
 
-from __future__ import with_statement
+
 
 
 import sys
@@ -30,17 +29,12 @@ import logging
 import types
 import collections
 import ast
+import json
 
-
-if sys.version_info >= (2, 6):
-	import json # in standard lib since 2.6
-else: #pragma: no cover
-	import simplejson as json # extra dependency
-
-
-from zim.signals import SignalEmitter, ConnectorMixin
-from zim.utils import OrderedDict, FunctionThread
+from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_NORMAL
+from zim.utils import OrderedDict
 from zim.fs import File, FileNotFoundError
+from zim.newfs import FileNotFoundError as NewFileNotFoundError
 from zim.errors import Error
 
 from .basedirs import XDG_CONFIG_HOME
@@ -49,17 +43,25 @@ from .basedirs import XDG_CONFIG_HOME
 logger = logging.getLogger('zim.config')
 
 
-class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
+class _MyMeta(type(SignalEmitter), type(OrderedDict)):
+	# Combine meta classes to resolve conflict
+	pass
+
+
+class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin, metaclass=_MyMeta):
 	'''Sub-class of C{OrderedDict} that tracks modified state.
 	This modified state is recursive for nested C{ControlledDict}s.
 
-	Used as base class for L{SectionedConfigDict}, L{ConfigDict}
-	and L{HeadersDict}.
+	Used as base class for L{SectionedConfigDict} and L{ConfigDict}.
 
 	@signal: C{changed ()}: emitted when content of this dict changed,
 	or a nested C{ControlledDict} changed
 	'''
 
+	__signals__ = {
+		'changed': (SIGNAL_NORMAL, None, ())
+	}
+
 	def __init__(self, E=None, **F):
 		OrderedDict.__init__(self, E, **F)
 		self._modified = False
@@ -80,7 +82,7 @@ class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 
 	def update(self, E=(), **F):
 		# Only emit changed once here
-		with self.blocked_signals('changed'):
+		with self.block_signals('changed'):
 			OrderedDict.update(self, E, **F)
 		self.emit('changed')
 
@@ -109,7 +111,7 @@ class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 			self._modified = True
 		else:
 			self._modified = False
-			for v in self.values():
+			for v in list(self.values()):
 				if isinstance(v, ControlledDict):
 					v.set_modified(False)
 
@@ -120,12 +122,11 @@ class ConfigDefinition(object):
 	__slots__ = ('default', 'allow_empty')
 
 	def __init__(self, default, allow_empty=False):
-		self.default = default
 		if default is None:
 			allow_empty = True
 		self.allow_empty = allow_empty
 
-		self.check(default) # ensure that default follows check
+		self.default = self.check(default) # ensure that default follows check
 
 	def __eq__(self, other):
 		return self.__class__ == other.__class__ \
@@ -139,7 +140,7 @@ class ConfigDefinition(object):
 			if self.allow_empty:
 				return True
 			else:
-				raise ValueError, 'Value not allowed to be empty'
+				raise ValueError('Value not allowed to be empty')
 		else:
 			return False
 
@@ -180,14 +181,14 @@ class ConfigDefinitionByClass(ConfigDefinition):
 	'''
 	# TODO fully get rid of this class and replace by specialized classes
 
-	__slots__= ('klass',)
+	__slots__ = ('klass',)
 
 	def __init__(self, default, klass=None, allow_empty=False):
 		if klass is None:
 			klass = default.__class__
 
-		if issubclass(klass, basestring):
-			self.klass = basestring
+		if issubclass(klass, str):
+			self.klass = str
 		else:
 			self.klass = klass
 
@@ -200,8 +201,8 @@ class ConfigDefinitionByClass(ConfigDefinition):
 	def check(self, value):
 		if self._check_allow_empty(value):
 			return None
-		elif isinstance(value, basestring) \
-		and not self.klass is basestring:
+		elif isinstance(value, str) \
+		and not self.klass is str:
 			value = self._eval_string(value)
 
 		if isinstance(value, self.klass):
@@ -215,15 +216,15 @@ class ConfigDefinitionByClass(ConfigDefinition):
 				return self.klass.new_from_zim_config(value)
 			except:
 				logger.debug('Error while converting %s to %s', value, self.klass, exc_info=1)
-				raise ValueError, 'Can not convert %s to %s' % (value, self.klass)
+				raise ValueError('Can not convert %s to %s' % (value, self.klass))
 		else:
-			raise ValueError, 'Value should be of type: %s' % self.klass.__name__
+			raise ValueError('Value should be of type: %s' % self.klass.__name__)
 
 	def tostring(self, value):
 		if hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			return json.dumps(value, separators=(',',':'))
+			return json.dumps(value, separators=(',', ':'))
 				# specify separators for compact encoding
 
 
@@ -238,7 +239,7 @@ class Boolean(ConfigDefinition):
 		elif value in ('True', 'true', 'False', 'false'):
 			return value in ('True', 'true')
 		else:
-			raise ValueError, 'Must be True or False'
+			raise ValueError('Must be True or False')
 
 
 class String(ConfigDefinition):
@@ -254,12 +255,12 @@ class String(ConfigDefinition):
 	def check(self, value):
 		if self._check_allow_empty(value):
 			return None
-		elif isinstance(value, basestring):
+		elif isinstance(value, str):
 			return value
 		elif hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			raise ValueError, 'Must be string'
+			raise ValueError('Must be string')
 
 	def tostring(self, value):
 		if value is None:
@@ -289,7 +290,7 @@ class Integer(ConfigDefinition):
 			try:
 				return int(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Float(ConfigDefinition):
@@ -304,7 +305,7 @@ class Float(ConfigDefinition):
 			try:
 				return float(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Choice(ConfigDefinition):
@@ -315,8 +316,8 @@ class Choice(ConfigDefinition):
 	__slots__ = ('choices',)
 
 	# TODO - this class needs a type for the choices
-	#        could be simply commen type of list items, but we get
-	#        bitten because we allow tuples as needed for preferences
+	#        could be simply common type of list items, but we get
+	#        beaten because we allow tuples as needed for preferences
 	#        with label --> make that a dedicated feature
 
 	def __init__(self, default, choices, allow_empty=False):
@@ -332,12 +333,12 @@ class Choice(ConfigDefinition):
 			return None
 		else:
 			# Allow options that are not strings (e.g. tuples of strings)
-			if isinstance(value, basestring) \
-			and not all(isinstance(t, basestring) for t in self.choices):
+			if isinstance(value, str) \
+			and not all(isinstance(t, str) for t in self.choices):
 				value = self._eval_string(value)
 
 			# HACK to allow for preferences with "choice" item that has
-			# a list of tuples as argumnet
+			# a list of tuples as argument
 			if all(isinstance(t, tuple) for t in self.choices):
 				choices = list(self.choices) + [t[0] for t in self.choices]
 			else:
@@ -350,10 +351,10 @@ class Choice(ConfigDefinition):
 
 			if value in choices:
 				return value
-			elif isinstance(value, basestring) and value.lower() in choices:
+			elif isinstance(value, str) and value.lower() in choices:
 				return value.lower()
 			else:
-				raise ValueError, 'Value should be one of %s' % unicode(choices)
+				raise ValueError('Value should be one of %s' % str(choices))
 
 
 class Range(Integer):
@@ -379,7 +380,7 @@ class Range(Integer):
 		elif self.min <= value <= self.max:
 			return value
 		else:
-			raise ValueError, 'Value should be between %i and %i' % (self.min, self.max)
+			raise ValueError('Value should be between %i and %i' % (self.min, self.max))
 
 
 class Coordinate(ConfigDefinition):
@@ -391,11 +392,11 @@ class Coordinate(ConfigDefinition):
 
 	def __init__(self, default, allow_empty=False):
 		if default == (None, None):
-			allow_empty=True
+			allow_empty = True
 		ConfigDefinition.__init__(self, default, allow_empty)
 
 	def check(self, value):
-		if isinstance(value, basestring):
+		if isinstance(value, str):
 			value = self._eval_string(value)
 
 		if self._check_allow_empty(value) \
@@ -413,15 +414,13 @@ class Coordinate(ConfigDefinition):
 			):
 				return value
 			else:
-				raise ValueError, 'Value should be a coordinate (tuple of 2 integers)'
+				raise ValueError('Value should be a coordinate (tuple of 2 integers)')
 
 value_is_coord = Coordinate # XXX for backward compatibility
 
 
 _definition_classes = {
 	str: String,
-	unicode: String,
-	basestring: String,
 	int: Integer,
 	float: Float,
 	bool: Boolean,
@@ -430,14 +429,14 @@ _definition_classes = {
 
 def build_config_definition(default=None, check=None, allow_empty=False):
 	'''Convenience method to construct a L{ConfigDefinition} object
-	based on a default value an/or a check.
+	based on a default value and/or a check.
 	'''
 	if default is None and check is None:
-		raise AssertionError, 'At least provide either a default or a check'
+		raise AssertionError('At least provide either a default or a check')
 	elif check is None:
 		check = default.__class__
 
-	if isinstance(check, (type, types.ClassType)): # is a class
+	if isinstance(check, type): # is a class
 		if issubclass(check, ConfigDefinition):
 			return check(default, allow_empty=allow_empty)
 		elif check in _definition_classes:
@@ -453,7 +452,7 @@ def build_config_definition(default=None, check=None, allow_empty=False):
 			and isinstance(check[1], int)
 		return Range(default, check[0], check[1])
 	else:
-		raise ValueError, 'Unrecognized check type'
+		raise ValueError('Unrecognized check type')
 
 
 
@@ -480,10 +479,9 @@ class ConfigDict(ControlledDict):
 		assert not (E and F)
 		ControlledDict.__init__(self)
 		self.definitions = OrderedDict()
-		if E:
-			self._input = dict(E)
-		else:
-			self._input = F
+		self._input = {}
+		if E or F:
+			self.input(E or F)
 
 	def copy(self):
 		'''Shallow copy of the items
@@ -492,6 +490,7 @@ class ConfigDict(ControlledDict):
 		new = self.__class__()
 		new.update(self)
 		new._input.update(self._input)
+		new._keys[:] = list(self._keys)
 		return new
 
 	def update(self, E=None, **F):
@@ -507,19 +506,47 @@ class ConfigDict(ControlledDict):
 			)
 		ControlledDict.update(self, E, **F)
 
+	def dump(self):
+		'''Returns a dict that combines the defined keys with any
+		undefined input keys. Used e.g. when you only define part of the
+		keys in the dict, but want to preserve all of them when
+		writing back to a file.
+		'''
+		return dict(self.all_items()) # FIXME should be OrderedDict, but causes test errors
+
+	def all_items(self):
+		# Like items() but returns both defined values and input values
+		for k in self._keys:
+			if k in self._values:
+				yield k, self._values[k]
+			elif k in self._input:
+				yield k, self._input[k]
+			else:
+				pass
+
 	# Note that OrderedDict optimizes __getitem__, cannot overload it
 
 	def __setitem__(self, k, v):
 		if k in self.definitions:
 			try:
 				v = self.definitions[k].check(v)
-			except ValueError, error:
-				raise ValueError, 'Invalid config value for %s: "%s" - %s' % (k, v, error.args[0])
+			except ValueError as error:
+				raise ValueError('Invalid config value for %s: "%s" - %s' % (k, v, error.args[0]))
 			else:
 				ControlledDict.__setitem__(self, k, v)
 		else:
 			raise KeyError('Config key "%s" has not been defined' % k)
 
+	def __delitem__(self, k):
+		if k in self._values:
+			ControlledDict.__delitem__(self, k)
+		else:
+			del self._input[k]
+			try:
+				self._keys.remove(k)
+			except ValueError:
+				pass
+
 	def input(self, E=None, **F):
 		'''Like C{update()} but won't raise on failures.
 		Values for undefined keys are stored and validated once the
@@ -528,8 +555,8 @@ class ConfigDict(ControlledDict):
 		'''
 		assert not (E and F)
 		update = E or F
-		if isinstance(update, collections.Mapping):
-			items = update.items()
+		if hasattr(update, 'items'):
+			items = list(update.items())
 		else:
 			items = update
 
@@ -538,25 +565,26 @@ class ConfigDict(ControlledDict):
 				self._set_input(key, value)
 			else:
 				self._input[key] = value # validated later
+				if key not in self._keys:
+					self._keys.append(key)
 
 	def define(self, E=None, **F):
-		'''Set one or more defintions for this config dict
+		'''Set one or more definitions for this config dict
 		Can cause error log when values prior given to C{input()} do
 		not match the definition.
 		'''
 		assert not (E and F)
 		update = E or F
 		if isinstance(update, collections.Mapping):
-			items = update.items()
+			items = list(update.items())
 		else:
 			items = update
 
 		for key, definition in items:
 			if key in self.definitions:
 				if definition != self.definitions[key]:
-					raise AssertionError, \
-						'Key is already defined with different definition: %s\n%s != %s' \
-						% (key, definition, self.definitions[key])
+					raise AssertionError('Key is already defined with different definition: %s\n%s != %s'
+						% (key, definition, self.definitions[key]))
 				else:
 					continue
 
@@ -565,20 +593,20 @@ class ConfigDict(ControlledDict):
 				value = self._input.pop(key)
 				self._set_input(key, value)
 			else:
-				with self.blocked_signals('changed'):
+				with self.block_signals('changed'):
 					OrderedDict.__setitem__(self, key, definition.default)
 
 	def _set_input(self, key, value):
 		try:
 			value = self.definitions[key].check(value)
-		except ValueError, error:
+		except ValueError as error:
 			logger.warn(
 				'Invalid config value for %s: "%s" - %s',
 					key, value, error.args[0]
 			)
 			value = self.definitions[key].default
 
-		with self.blocked_signals('changed'):
+		with self.block_signals('changed'):
 			OrderedDict.__setitem__(self, key, value)
 
 	def setdefault(self, key, default, check=None, allow_empty=False):
@@ -661,7 +689,7 @@ class ConfigDict(ControlledDict):
 
 class SectionedConfigDict(ControlledDict):
 	'''Dict with multiple sections of config values
-	Sections are auto-vivicated when a non-existing item is retrieved.
+	Sections are handled automatically when a non-existing item is retrieved.
 	'''
 
 	def __setitem__(self, k, v):
@@ -672,14 +700,14 @@ class SectionedConfigDict(ControlledDict):
 		try:
 			return ControlledDict.__getitem__(self, k)
 		except KeyError:
-			with self.blocked_signals('changed'):
+			with self.block_signals('changed'):
 				ControlledDict.__setitem__(self, k, ConfigDict())
 			return ControlledDict.__getitem__(self, k)
 
 
 class INIConfigFile(SectionedConfigDict):
 	'''Dict to represent a configuration file in "ini-style". Since the
-	ini-file is devided in section this is represented as a dict of
+	ini-file is divided in section this is represented as a dict of
 	dicts. This class represents the top-level with a key for each
 	section. The values are in turn L{ConfigDict}s which contain the
 	key value pairs in that section.
@@ -723,10 +751,10 @@ class INIConfigFile(SectionedConfigDict):
 		SectionedConfigDict.__init__(self)
 		self.file = file
 		try:
-			with self.blocked_signals('changed'):
+			with self.block_signals('changed'):
 				self.read()
 			self.set_modified(False)
-		except FileNotFoundError:
+		except (FileNotFoundError, NewFileNotFoundError):
 			pass
 
 		if monitor:
@@ -735,15 +763,15 @@ class INIConfigFile(SectionedConfigDict):
 	def on_file_changed(self, *a):
 		if self.file.check_has_changed_on_disk():
 			try:
-				with self.blocked_signals('changed'):
+				with self.block_signals('changed'):
 					self.read()
-			except FileNotFoundError:
+			except (FileNotFoundError, NewFileNotFoundError):
 				pass
 			else:
 				# First emit top level to allow general changes
 				self.emit('changed')
-				with self.blocked_signals('changed'):
-					for section in self.values():
+				with self.block_signals('changed'):
+					for section in list(self.values()):
 						section.emit('changed')
 				self.set_modified(False)
 
@@ -763,7 +791,7 @@ class INIConfigFile(SectionedConfigDict):
 		# Note that we explicitly do _not_ support comments on the end
 		# of a line. This is because "#" could be a valid character in
 		# a config value.
-		if isinstance(text, basestring):
+		if isinstance(text, str):
 			text = text.splitlines(True)
 
 		section = None
@@ -797,35 +825,26 @@ class INIConfigFile(SectionedConfigDict):
 		self.file.writelines(self.dump())
 		self.set_modified(False)
 
-	def write_async(self):
-		'''Write data asynchronously and set C{modified} to C{False}
-		@returns: an L{FunctionThread} object
-		'''
-		func = FunctionThread(
-			self.file,
-			self.file.writelines,
-			self.dump())
-		func.start()
-		self.set_modified(False)
-		return func
-
 	def dump(self):
 		'''Serialize the config to a "ini-style" config file.
 		@returns: a list of lines with text in "ini-style" formatting
 		'''
 		lines = []
 		def dump_section(name, section):
-			try:
-				lines.append('[%s]\n' % name)
-				for key, value in section.items():
-					if not key.startswith('_'):
-						lines.append('%s=%s\n' % (key, section.definitions[key].tostring(value)))
-				lines.append('\n')
-			except:
-				logger.exception('Dumping section [%s] failed:\n%r', name, section)
+			lines.append('[%s]\n' % name)
+			for key, value in section.all_items():
+				if not key.startswith('_'):
+					try:
+						if key in section.definitions:
+							lines.append('%s=%s\n' % (key, section.definitions[key].tostring(value)))
+						else:
+							lines.append('%s=%s\n' % (key, value))
+					except:
+						logger.exception('Error serializing "%s" in section "[%s]"', key, name)
+			lines.append('\n')
 
-		for name, section in self.items():
-			if section and not name.startswith('_'):
+		for name, section in list(self.items()):
+			if not name.startswith('_'):
 				if isinstance(section, list):
 					for s in section:
 						dump_section(name, s)
@@ -835,114 +854,8 @@ class INIConfigFile(SectionedConfigDict):
 		return lines
 
 
-class HeaderParsingError(Error):
-	'''Error when parsing a L{HeadersDict}'''
-
-	description = '''\
-Invalid data was found in a block with headers.
-This probably means the header block was corrupted
-and can not be read correctly.'''
-
-	def __init__(self, line):
-		self.msg = 'Invalid header >>%s<<' % line.strip('\n')
-
-
-class HeadersDict(ControlledDict):
-	'''This class maps a set of headers in the rfc822 format.
-	Can e.g. look like::
-
-		Content-Type: text/x-zim-wiki
-		Wiki-Format: zim 0.4
-		Creation-Date: 2010-12-14T14:15:09.134955
-
-	Header names are always kept in "title()" format to ensure
-	case-insensitivity.
-	'''
-
-	_is_header_re = re.compile('^([\w\-]+):\s+(.*)')
-	_is_continue_re = re.compile('^(\s+)(?=\S)')
-
-	def __init__(self, text=None):
-		'''Constructor
-
-		@param text: the header text, passed on to L{parse()}
-		'''
-		OrderedDict.__init__(self)
-		if not text is None:
-			self.parse(text)
-
-	def __getitem__(self, k):
-		return OrderedDict.__getitem__(self, k.title())
-
-	def __setitem__(self, k, v):
-		return OrderedDict.__setitem__(self, k.title(), v)
-
-	def read(self, lines):
-		'''Checks for headers at the start of the list of lines and
-		read them into the dict until the first empty line. Will remove
-		any lines belonging to the header block from the original list,
-		so after this method returns the input does no longer contain
-		the header block.
-		@param lines: a list of lines
-		'''
-		self._parse(lines, fatal=False)
-		if lines and lines[0].isspace():
-			lines.pop(0)
-
-	def parse(self, text):
-		'''Adds headers defined in 'text' to the dict.
-		Trailing whitespace is ignored.
-
-		@param text: a header block, either as string or as a list of lines.
-
-		@raises HeaderParsingError: when C{text} is not a valid header
-		block
-		'''
-		if isinstance(text, basestring):
-			lines = text.rstrip().splitlines(True)
-		else:
-			lines = text[:] # make copy so we do not destry the original
-		self._parse(lines)
-
-	def _parse(self, lines, fatal=True):
-		header = None
-		while lines:
-			is_header = self._is_header_re.match(lines[0])
-			if is_header:
-				header = is_header.group(1)
-				value  = is_header.group(2)
-				self[header] = value.strip()
-			elif self._is_continue_re.match(lines[0]) and not header is None:
-				self[header] += '\n' + lines[0].strip()
-			else:
-				if fatal:
-					raise HeaderParsingError, lines[0]
-				else:
-					break
-			lines.pop(0)
-
-	def dump(self, strict=False):
-		'''Serialize the dict to a header block in rfc822 header format.
-
-		@param strict: if C{True} lines will be properly terminated
-		with '\\r\\n' instead of '\\n'.
-
-		@returns: the header block as a list of lines
-		'''
-		buffer = []
-		for k, v in self.items():
-			v = v.strip().replace('\n', '\n\t')
-			buffer.extend((k, ': ', v, '\n'))
-		text = ''.join(buffer)
-
-		if strict:
-			text = text.replace('\n', '\r\n')
-
-		return text.splitlines(True)
-
-
 class HierarchicDict(object):
-	'''This class implements a data store that behaves as a hierarchig
+	'''This class implements a data store that behaves as a hierarchic
 	dict of dicts. Each key in this object is considered a hierarchic
 	path (the path separator is ':' for obvious reasons). The dict for
 	each key will "inherit" all values from parent paths. However
@@ -955,8 +868,8 @@ class HierarchicDict(object):
 	There is a special member dict stored under the key "__defaults__"
 	which has the top-level fallback properties.
 
-	Child dicts are auto-vivicated, so this object only implements
-	C{__getitem__()} but no C{__setitem__()}.
+	This object only implements
+	C{__getitem__()}, but no C{__setitem__()}.
 	'''
 	# Note that all the magic is actually implemented by HierarchicDictFrame
 
@@ -971,7 +884,7 @@ class HierarchicDict(object):
 		self.dict['__defaults__'] = defaults or {}
 
 	def __getitem__(self, k):
-		if not isinstance(k, basestring):
+		if not isinstance(k, str):
 			k = k.name # assume zim path
 		return HierarchicDictFrame(self.dict, k)
 
