@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -28,8 +27,8 @@ Supported tags:
 	- pre for verbatim paragraphs (no further parsing in these blocks)
 	- em for emphasis, rendered italic by default
 	- strong for strong emphasis, rendered bold by default
-	- mark for highlighted text, renderd with background color or underlined
-	- strike for text that is removed, usually renderd as strike through
+	- mark for highlighted text, rendered with background color or underlined
+	- strike for text that is removed, usually rendered as strike through
 	- code for inline verbatim text
 	- ul for bullet and checkbox lists
 	- ol for numbered lists
@@ -67,6 +66,7 @@ to a title or subtitle in the document.
 
 import re
 import string
+import itertools
 import logging
 
 import types
@@ -76,9 +76,10 @@ from zim.parsing import link_type, is_url_re, \
 	url_encode, url_decode, URL_ENCODE_READABLE, URL_ENCODE_DATA
 from zim.parser import Builder
 from zim.config import data_file, ConfigDict
-from zim.objectmanager import ObjectManager
+from zim.plugins import PluginManager
 
 import zim.plugins
+from functools import reduce
 
 
 logger = logging.getLogger('zim.formats')
@@ -86,9 +87,9 @@ logger = logging.getLogger('zim.formats')
 # Needed to determine RTL, but may not be available
 # if gtk bindings are not installed
 try:
-	import pango
+	from gi.repository import Pango
 except:
-	pango = None
+	Pango = None
 	logger.warn('Could not load pango - RTL scripts may look bad')
 
 try:
@@ -106,6 +107,7 @@ TEXT_FORMAT = 8 # Used for "Copy As" menu - these all prove "text/plain" mimetyp
 UNCHECKED_BOX = 'unchecked-box'
 CHECKED_BOX = 'checked-box'
 XCHECKED_BOX = 'xchecked-box'
+MIGRATED_BOX = 'migrated-box'
 BULLET = '*' # FIXME make this 'bullet'
 
 FORMATTEDTEXT = 'zim-tree'
@@ -141,9 +143,12 @@ HEADDATA = 'th'
 TABLEROW = 'trow'
 TABLEDATA = 'td'
 
+LINE = 'line'
+
 BLOCK_LEVEL = (PARAGRAPH, HEADING, VERBATIM_BLOCK, BLOCK, OBJECT, IMAGE, LISTITEM, TABLE)
 
 
+
 _letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
 
 def increase_list_iter(listiter):
@@ -160,7 +165,7 @@ def increase_list_iter(listiter):
 	except ValueError:
 		try:
 			i = _letters.index(listiter)
-			return _letters[i+1]
+			return _letters[i + 1]
 		except ValueError: # listiter is not a letter
 			return None
 		except IndexError: # wrap to start of list
@@ -176,7 +181,7 @@ def encode_xml(text):
 
 def list_formats(type):
 	if type == EXPORT_FORMAT:
-		return ['HTML','LaTeX', 'Markdown (pandoc)', 'RST (sphinx)']
+		return ['HTML', 'LaTeX', 'Markdown (pandoc)', 'RST (sphinx)']
 	elif type == TEXT_FORMAT:
 		return ['Text', 'Wiki', 'Markdown (pandoc)', 'RST (sphinx)']
 	else:
@@ -190,14 +195,16 @@ def canonical_name(name):
 	name = name.lower()
 	if ' ' in name:
 		name, _ = name.split(' ', 1)
-	if name == 'text': return 'plain'
-	else: return name
+	if name == 'text':
+		return 'plain'
+	else:
+		return name
 
 
 def get_format(name):
 	'''Returns the module object for a specific format.'''
 	# If this method is removes, class names in formats/*.py can be made more explicit
-	#~ print 'DEPRECATED: get_format() is deprecated in favor if get_parser() and get_dumper()'
+	#~ print('DEPRECATED: get_format() is deprecated in favor if get_parser() and get_dumper()')
 	return get_format_module(name)
 
 
@@ -241,7 +248,7 @@ def get_dumper(name, *arg, **kwarg):
 class ParseTree(object):
 	'''Wrapper for zim parse trees.'''
 
-	# No longer derives from ElementTree, internals are not private
+	# No longer derives from ElementTree, internals are now private
 
 	# TODO, also remove etree args from init
 	# TODO, rename to FormattedText
@@ -249,12 +256,15 @@ class ParseTree(object):
 	def __init__(self, *arg, **kwarg):
 		self._etree = ElementTreeModule.ElementTree(*arg, **kwarg)
 		self._object_cache = {}
+		self.meta = OrderedDict()
 
 	@property
 	def hascontent(self):
 		'''Returns True if the tree contains any content at all.'''
 		root = self._etree.getroot()
-		return bool(root.getchildren()) or (root.text and not root.text.isspace())
+		return root is not None and (
+			bool(root.getchildren()) or (root.text and not root.text.isspace())
+		)
 
 	@property
 	def ispartial(self):
@@ -291,7 +301,7 @@ class ParseTree(object):
 
 	def fromstring(self, string):
 		'''Set the contents of this tree from XML representation.'''
-		parser = ElementTreeModule.XMLTreeBuilder()
+		parser = ElementTreeModule.XMLParser()
 		parser.feed(string)
 		root = parser.close()
 		self._etree._setroot(root)
@@ -299,26 +309,59 @@ class ParseTree(object):
 
 	def tostring(self):
 		'''Serialize the tree to a XML representation'''
-		from cStringIO import StringIO
+		from io import StringIO
 
 		# Parent dies when we have attributes that are not a string
 		for element in self._etree.getiterator('*'):
-			for key in element.attrib.keys():
+			for key in list(element.attrib.keys()):
 				element.attrib[key] = str(element.attrib[key])
 
 		xml = StringIO()
 		xml.write("<?xml version='1.0' encoding='utf-8'?>\n")
-		ElementTreeModule.ElementTree.write(self._etree, xml, 'utf-8')
+		ElementTreeModule.ElementTree.write(self._etree, xml, 'unicode')
 		return xml.getvalue()
 
 	def copy(self):
-		# By using serialization we are absolutely sure all refs are new
-		xml = self.tostring()
-		try:
-			return ParseTree().fromstring(xml)
-		except:
-			print ">>>", xml, "<<<"
-			raise
+		builder = ParseTreeBuilder(_parsetree_roundtrip=True)
+		self.visit(builder)
+		return builder.get_parsetree()
+
+	def iter_tokens(self):
+		from zim.tokenparser import TokenBuilder
+
+		tb = TokenBuilder()
+		self.visit(tb)
+		return iter(tb.tokens)
+
+	def iter_href(self):
+		'''Generator for links in the text
+		@returns: yields a list of unique L{HRef} objects
+		'''
+		from zim.notebook.page import HRef # XXX
+		seen = set()
+		for elt in itertools.chain(
+			self._etree.getiterator(LINK),
+			self._etree.getiterator(IMAGE)
+		):
+			href = elt.attrib.get('href')
+			if href and href not in seen:
+				seen.add(href)
+				if link_type(href) == 'page':
+					try:
+						yield HRef.new_from_wiki_link(href)
+					except ValueError:
+						pass
+
+	def iter_tag_names(self):
+		'''Generator for tags in the page content
+		@returns: yields an unordered list of tag names
+		'''
+		seen = set()
+		for elt in self._etree.getiterator(TAG):
+			name = elt.text
+			if not name in seen:
+				seen.add(name)
+				yield name.lstrip('@')
 
 	def _get_heading_element(self, level=1):
 		root = self._etree.getroot()
@@ -328,7 +371,7 @@ class ParseTree(object):
 
 		if children:
 			first = children[0]
-			if first.tag == 'h' and first.attrib['level'] >= level:
+			if first.tag == 'h' and int(first.attrib['level']) >= level:
 				return first
 		return None
 
@@ -358,7 +401,7 @@ class ParseTree(object):
 			root = self._etree.getroot()
 			heading = ElementTreeModule.Element('h', {'level': level})
 			heading.text = text
-			heading.tail = root.text
+			heading.tail = '\n' + (root.text or '')
 			root.text = None
 			root.insert(0, heading)
 
@@ -402,7 +445,7 @@ class ParseTree(object):
 			while path and path[-1][0] >= level:
 				path.pop()
 			if not path:
-				newlevel = offset+1
+				newlevel = offset + 1
 			else:
 				newlevel = path[-1][1] + 1
 			if newlevel > max:
@@ -609,7 +652,7 @@ class ParseTree(object):
 				elt.remove(child)
 				offset -= 1
 				for item in node:
-					if isinstance(item, basestring):
+					if isinstance(item, str):
 						self._insert_text(elt, i, item)
 					else:
 						assert isinstance(item, Element)
@@ -619,7 +662,7 @@ class ParseTree(object):
 				if tail:
 					self._insert_text(elt, i, tail)
 			else:
-				raise TypeError, 'BUG: invalid replacement result'
+				raise TypeError('BUG: invalid replacement result')
 
 	@staticmethod
 	def _node_to_etree(node):
@@ -634,35 +677,12 @@ class ParseTree(object):
 			else:
 				elt.text = text
 		else:
-			prev = elt[i-1]
+			prev = elt[i - 1]
 			if prev.tail:
 				prev.tail += text
 			else:
 				prev.tail = text
 
-	def get_objects(self, type=None):
-		'''Generator that yields all custom objects in the tree,
-		or all objects of a certain type.
-		@param type: object type to return or C{None} to get all
-		@returns: yields objects (as provided by L{ObjectManager})
-		'''
-		for elt in self._etree.getiterator(OBJECT):
-			if type and elt.attrib.get('type') != type:
-				pass
-			else:
-				obj = self._get_object(elt)
-				if obj is not None:
-					yield obj
-
-	def _get_object(self, elt):
-		## TODO optimize using self._object_cache or new API for
-		## passing on objects in the tree
-		type = elt.attrib.get('type')
-		if elt.tag == OBJECT and type:
-			return ObjectManager.get_object(type, elt.attrib, elt.text)
-		else:
-			return None
-
 
 class VisitorStop(Exception):
 	'''Exception to be raised to cancel a visitor action'''
@@ -742,11 +762,12 @@ class Visitor(object):
 class ParseTreeBuilder(Builder):
 	'''Builder object that builds a L{ParseTree}'''
 
-	def __init__(self, partial=False):
+	def __init__(self, partial=False, _parsetree_roundtrip=False):
 		self.partial = partial
 		self._b = ElementTreeModule.TreeBuilder()
 		self.stack = [] #: keeps track of current open elements
 		self._last_char = None
+		self._parsetree_roundtrip = _parsetree_roundtrip
 
 	def get_parsetree(self):
 		'''Returns the constructed L{ParseTree} object.
@@ -759,6 +780,7 @@ class ParseTreeBuilder(Builder):
 		return zim.formats.ParseTree(root)
 
 	def start(self, tag, attrib=None):
+		attrib = attrib.copy() if attrib is not None else None
 		self._b.start(tag, attrib)
 		self.stack.append(tag)
 		if tag in BLOCK_LEVEL:
@@ -775,9 +797,9 @@ class ParseTreeBuilder(Builder):
 
 	def end(self, tag):
 		if tag != self.stack[-1]:
-			raise AssertionError, 'Unmatched tag closed: %s' % tag
+			raise AssertionError('Unmatched tag closed: %s' % tag)
 
-		if tag in BLOCK_LEVEL:
+		if tag in BLOCK_LEVEL and not self._parsetree_roundtrip:
 			if self._last_char is not None and not self.partial:
 				#~ assert self._last_char == '\n', 'Block level text needs to end with newline'
 				if self._last_char != '\n' and tag not in (HEADING, LISTITEM):
@@ -791,12 +813,13 @@ class ParseTreeBuilder(Builder):
 		self.stack.pop()
 
 		# FIXME hack for backward compat
-		if tag == HEADING:
+		if tag == HEADING and not self._parsetree_roundtrip:
 			self._b.data('\n')
 
 		self._last_char = None
 
 	def append(self, tag, attrib=None, text=None):
+		attrib = attrib.copy() if attrib is not None else None
 		if tag in BLOCK_LEVEL:
 			if text and not text.endswith('\n'):
 				text += '\n'
@@ -811,7 +834,7 @@ class ParseTreeBuilder(Builder):
 		self._b.end(tag)
 
 		# FIXME hack for backward compat
-		if tag == HEADING:
+		if tag == HEADING and not self._parsetree_roundtrip:
 			self._b.data('\n')
 
 		self._last_char = None
@@ -861,7 +884,7 @@ class OldParseTreeBuilder(object):
 			self._flush(need_eol=1)
 		else:
 			self._flush()
-		#~ print 'START', tag
+		#~ print('START', tag)
 
 		if tag == 'h':
 			if not (attrib and 'level' in attrib):
@@ -896,7 +919,7 @@ class OldParseTreeBuilder(object):
 			self._flush(need_eol=1)
 		else:
 			self._flush()
-		#~ print 'END', tag
+		#~ print('END', tag)
 
 		self._last = self._stack[-1]
 		assert self._last.tag == tag, \
@@ -933,7 +956,7 @@ class OldParseTreeBuilder(object):
 			return self._stack.pop()
 
 	def data(self, text):
-		assert isinstance(text, basestring)
+		assert isinstance(text, str)
 		self._data.append(text)
 
 	def append(self, tag, text):
@@ -944,14 +967,16 @@ class OldParseTreeBuilder(object):
 	def _flush(self, need_eol=0):
 		# need_eol makes sure previous data ends with \n
 
-		#~ print 'DATA:', self._data
+		#~ print('DATA:', self._data)
 		text = ''.join(self._data)
 
 		# Fix trailing newlines
 		if text:
 			m = count_eol_re.search(text)
-			if m: self._seen_eol = len(m.group(0))
-			else: self._seen_eol = 0
+			if m:
+				self._seen_eol = len(m.group(0))
+			else:
+				self._seen_eol = 0
 
 		if need_eol > self._seen_eol:
 			text += '\n' * (need_eol - self._seen_eol)
@@ -969,7 +994,7 @@ class OldParseTreeBuilder(object):
 		and text.startswith('\n'):
 			text = text[1:]
 			if not text.strip('\n'):
-				self._seen_eol -=1
+				self._seen_eol -= 1
 
 		if text:
 			assert not self._last is None, 'data seen before root element'
@@ -1058,9 +1083,9 @@ class ParserClass(object):
 		i = url.find('?')
 		if i > 0:
 			attrib = {'src': url[:i]}
-			for option in url[i+1:].split('&'):
+			for option in url[i + 1:].split('&'):
 				if option.find('=') == -1:
-					logger.warn('Mal-formed options in "%s"' , url)
+					logger.warn('Mal-formed options in "%s"', url)
 					break
 
 				k, v = option.split('=', 1)
@@ -1133,15 +1158,16 @@ class DumperClass(Visitor):
 		self._text = []
 
 	def dump(self, tree):
-		'''Convenience methods to dump a given tree.
+		'''Format a parsetree to text
 		@param tree: a parse tree object that supports a C{visit()} method
+		@returns: a list of lines
 		'''
 		# FIXME - issue here is that we need to reset state - should be in __init__
 		self._text = []
 		self.context = [DumperContextElement(None, None, self._text)]
 		tree.visit(self)
 		if len(self.context) != 1:
-			raise AssertionError, 'Unclosed tags on tree: %s' % self.context[-1].tag
+			raise AssertionError('Unclosed tags on tree: %s' % self.context[-1].tag)
 		#~ import pprint; pprint.pprint(self._text)
 		return self.get_lines() # FIXME - maybe just return text ?
 
@@ -1149,7 +1175,7 @@ class DumperClass(Visitor):
 		'''Return the dumped content as a list of lines
 		Should only be called after closing the top level element
 		'''
-		return u''.join(self._text).splitlines(1)
+		return ''.join(self._text).splitlines(1)
 
 	def start(self, tag, attrib=None):
 		if attrib:
@@ -1164,7 +1190,7 @@ class DumperClass(Visitor):
 
 	def end(self, tag):
 		if not tag or tag != self.context[-1].tag:
-			raise AssertionError, 'Unexpected tag closed: %s' % tag
+			raise AssertionError('Unexpected tag closed: %s' % tag)
 		_, attrib, strings = self.context.pop()
 
 		if tag in self.TAGS:
@@ -1176,15 +1202,15 @@ class DumperClass(Visitor):
 			pass
 		else:
 			try:
-				method = getattr(self, 'dump_'+tag)
+				method = getattr(self, 'dump_' + tag)
 			except AttributeError:
-				raise AssertionError, 'BUG: Unknown tag: %s' % tag
+				raise AssertionError('BUG: Unknown tag: %s' % tag)
 
 			strings = method(tag, attrib, strings)
 			#~ try:
 				#~ u''.join(strings)
 			#~ except:
-				#~ print "BUG: %s returned %s" % ('dump_'+tag, strings)
+				#~ print("BUG: %s returned %s" % ('dump_'+tag, strings))
 
 		if strings is not None:
 			self.context[-1].text.extend(strings)
@@ -1204,9 +1230,9 @@ class DumperClass(Visitor):
 				attrib = attrib.copy() # Ensure dumping does not change tree
 
 			try:
-				method = getattr(self, 'dump_'+tag)
+				method = getattr(self, 'dump_' + tag)
 			except AttributeError:
-				raise AssertionError, 'BUG: Unknown tag: %s' % tag
+				raise AssertionError('BUG: Unknown tag: %s' % tag)
 
 			if text is None:
 				strings = method(tag, attrib, [])
@@ -1239,24 +1265,32 @@ class DumperClass(Visitor):
 		@param strings: a list of pieces of text
 		@returns: a new list of lines, each starting with prefix
 		'''
-		lines = u''.join(strings).splitlines(1)
+		lines = ''.join(strings).splitlines(1)
 		return [prefix + l for l in lines]
 
-	def dump_object(self, tag, attrib, strings=None):
-		'''Dumps object using proper ObjectManager'''
+	def dump_object(self, tag, attrib, strings=[]):
+		'''Dumps objects defined by L{InsertedObjectType}'''
 		format = str(self.__class__.__module__).split('.')[-1]
-		if 'type' in attrib:
-			obj = ObjectManager.get_object(attrib['type'], attrib, u''.join(strings))
-			output = obj.dump(format, self, self.linker)
-			if isinstance(output, basestring):
-				return [output]
-			elif output is not None:
+		try:
+			obj = PluginManager.insertedobjects[attrib['type']]
+		except KeyError:
+			pass
+		else:
+			try:
+				output = obj.format(format, self, attrib, ''.join(strings))
+			except ValueError:
+				pass
+			else:
+				assert isinstance(output, (list, tuple)), "Invalid output: %r" % output
 				return output
 
-		return self.dump_object_fallback(tag, attrib, strings)
-
-		# TODO put content in attrib, use text for caption (with full recursion)
-		# See img
+		if attrib['type'].startswith('image+'):
+			# Fallback for backward compatibility of image generators < zim 0.70
+			attrib = attrib.copy()
+			attrib['type'] = attrib['type'][6:]
+			return self.dump_img(IMAGE, attrib, None)
+		else:
+			return self.dump_object_fallback(tag, attrib, strings)
 
 	def dump_object_fallback(self, tag, attrib, strings=None):
 		'''Method to serialize objects that do not have their own
@@ -1271,22 +1305,22 @@ class DumperClass(Visitor):
 		@returns: C{True} if C{text} starts with characters in a
 		RTL script, or C{None} if direction is not determined.
 		'''
-		if pango is None:
+		if Pango is None:
 			return None
 
 		# It seems the find_base_dir() function is not documented in the
 		# python language bindings. The Gtk C code shows the signature:
 		#
-		#     pango.find_base_dir(text, length)
+		#     Pango.find_base_dir(text, length)
 		#
 		# It either returns a direction, or NEUTRAL if e.g. text only
 		# contains punctuation but no real characters.
 
-		dir = pango.find_base_dir(text, len(text))
-		if dir == pango.DIRECTION_NEUTRAL:
+		dir = Pango.find_base_dir(text, len(text))
+		if dir == Pango.Direction.NEUTRAL:
 			return None
 		else:
-			return dir == pango.DIRECTION_RTL
+			return dir == Pango.Direction.RTL
 
 
 class BaseLinker(object):
@@ -1443,12 +1477,12 @@ class Node(list):
 		@returns: string
 		'''
 		strings = self._gettext()
-		return u''.join(strings)
+		return ''.join(strings)
 
 	def _gettext(self):
 		strings = []
 		for item in self:
-			if isinstance(item, basestring):
+			if isinstance(item, str):
 				strings.append(item)
 			else:
 				strings.extend(item._gettext())
@@ -1456,7 +1490,7 @@ class Node(list):
 
 	def toxml(self):
 		strings = self._toxml()
-		return u''.join(strings)
+		return ''.join(strings)
 
 	def _toxml(self):
 		strings = []
@@ -1469,7 +1503,7 @@ class Node(list):
 			strings.append("<%s>" % self.tag)
 
 		for item in self:
-			if isinstance(item, basestring):
+			if isinstance(item, str):
 				strings.append(encode_xml(item))
 			else:
 				strings.extend(item._toxml())
@@ -1480,12 +1514,12 @@ class Node(list):
 	__repr__ = toxml
 
 	def visit(self, visitor):
-		if len(self) == 1 and isinstance(self[0], basestring):
+		if len(self) == 1 and isinstance(self[0], str):
 			visitor.append(self.tag, self.attrib, self[0])
 		else:
 			visitor.start(self.tag, self.attrib)
 			for item in self:
-				if isinstance(item, basestring):
+				if isinstance(item, str):
 					visitor.text(item)
 				else:
 					item.visit(visitor)
@@ -1517,7 +1551,7 @@ class TableParser():
 		:param lines: 2-dim multiline rows
 		:return: the number of characters of the longest cell-value by column
 		'''
-		widths = [max(map(len, line)) for line in zip(*lines)]
+		widths = [max(list(map(len, line))) for line in zip(*lines)]
 		return widths
 
 	@staticmethod
@@ -1527,23 +1561,23 @@ class TableParser():
 		:param lines: 3-dim multiline rows
 		:return: the number of characters of the longest cell-value by column
 		'''
-		lines = reduce(lambda x, y: x+y, lines)
-		widths = [max(map(len, line)) for line in zip(*lines)]
+		lines = reduce(lambda x, y: x + y, lines)
+		widths = [max(list(map(len, line))) for line in zip(*lines)]
 		return widths
 
 	@staticmethod
 	def convert_to_multiline_cells(rows):
 		'''
-		Each cell of a list of list is splitted by "\n" and a 3-dimensional list is returned,
+		Each cell in a list of rows is split by "\n" and a 3-dimensional list is returned,
 		whereas each tuple represents a line and multiple lines represents a row and multiple rows represents the table
 		c11a = Cell in Row 1 in Column 1 in first = a line
 		:param strings: format like (('c11a \n c11b', 'c12a \n c12b'), ('c21', 'c22a \n 22b'))
 		:return: format like (((c11a, c12a), (c11b, c12b)), ((c21, c22a), ('', c22b)))
 		'''
-		multi_rows = [map(lambda cell: cell.split("\n"), row) for row in rows]
+		multi_rows = [[cell.split("\n") for cell in row] for row in rows]
 
 		# grouping by line, not by row
-		strings = [map(lambda *line: map(lambda val: val if val is not None else '', line), *row) for row in multi_rows]
+		strings = [list(map(lambda *line: [val if val is not None else '' for val in line], *row)) for row in multi_rows]
 		return strings
 
 	@staticmethod
@@ -1554,7 +1588,7 @@ class TableParser():
 		:return: tuple of attributes
 		'''
 		aligns = attrib['aligns'].split(',')
-		wraps = map(int, attrib['wraps'].split(','))
+		wraps = list(map(int, attrib['wraps'].split(',')))
 
 		return aligns, wraps
 
@@ -1568,7 +1602,7 @@ class TableParser():
 		:param y: line-separator
 		:return: a textline
 		'''
-		return x + x.join(map(lambda width: (width+2) * y, maxwidths)) + x
+		return x + x.join([(width + 2) * y for width in maxwidths]) + x
 
 	@staticmethod
 	def headsep(maxwidths, aligns, x='|', y='-'):
@@ -1579,7 +1613,7 @@ class TableParser():
 		:param aligns:  list of alignments
 		:param x: point-separator
 		:param y: line-separator
-		:return: a textline
+		:return: a text line
 		'''
 		cells = []
 		for width, align in zip(maxwidths, aligns):
@@ -1610,7 +1644,7 @@ class TableParser():
 		cells = []
 		for val, wrap in zip(row, wraps):
 			if wrap == 1:
-				val = val[:-1]+'<'
+				val = val[:-1] + '<'
 			cells.append(val)
 		return x + x.join(cells) + x
 
@@ -1636,7 +1670,6 @@ class TableParser():
 		:param row: tuple of cells
 		:param maxwidths: list of column length
 		:param aligns:  list of alignments
-		:param x:  point-separator
 		:param y: space-separator
 		:return: a textline
 		'''
@@ -1647,9 +1680,69 @@ class TableParser():
 			elif align == 'right':
 				(lspace, rspace) = (maxwidth - len(val) + 1, 1)
 			elif align == 'center':
-				lspace = (maxwidth - len(val)) / 2 + 1
+				lspace = (maxwidth - len(val)) // 2 + 1
 				rspace = (maxwidth - lspace - len(val) + 2)
 			else:
 				(lspace, rspace) = (1, maxwidth - len(val) + 1)
 			cells.append(lspace * y + val + rspace * y)
 		return cells
+
+
+from zim.config.dicts import OrderedDict
+
+_is_header_re = re.compile('^([\w\-]+):\s+(.*?)\n', re.M)
+_is_continue_re = re.compile('^([^\S\n]+)(.+?)\n', re.M)
+
+def parse_header_lines(text):
+	'''Read header lines in the rfc822 format.
+	Can e.g. look like::
+
+		Content-Type: text/x-zim-wiki
+		Wiki-Format: zim 0.4
+		Creation-Date: 2010-12-14T14:15:09.134955
+
+	@returns: the text minus the headers and a dict with the headers
+	'''
+	assert isinstance(text, str)
+	meta = OrderedDict()
+	match = _is_header_re.match(text)
+	pos = 0
+	while match:
+		header = match.group(1)
+		value = match.group(2)
+		pos = match.end()
+
+		meta[header] = value.strip()
+		match = _is_continue_re.match(text, pos)
+		while match:
+			cont = match.group(2)
+			meta[header] += '\n' + cont.strip()
+			pos = match.end()
+			match = _is_continue_re.match(text, pos)
+
+		match = _is_header_re.match(text, pos)
+	else:
+		if pos > 0:
+			try:
+				if text[pos] == '\n':
+					pos += 1
+			except IndexError:
+				pass
+			text = text[pos:]
+
+	return text, meta
+
+
+def dump_header_lines(*headers):
+	'''Return text representation of header dict'''
+	text = []
+	append = lambda k, v: text.extend((k, ': ', v.strip().replace('\n', '\n\t'), '\n'))
+
+	for h in headers:
+		if hasattr(h, 'items'):
+			for k, v in list(h.items()):
+				append(k, v)
+		else:
+			for k, v in h:
+				append(k, v)
+	return ''.join(text)
