@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -40,14 +39,14 @@ def html_encode(text):
 class Dumper(DumperClass):
 
 	TAGS = {
-		EMPHASIS:		('<i>', '</i>'),
-		STRONG: 		('<b>', '</b>'),
-		MARK:			('<u>', '</u>'),
-		STRIKE: 		('<s>', '</s>'),
-		VERBATIM:		('<tt>', '</tt>'),
-		TAG:			('<span class="zim-tag">', '</span>'),
-		SUBSCRIPT:		('<sub>', '</sub>'),
-		SUPERSCRIPT:	('<sup>', '</sup>'),
+		EMPHASIS: ('<i>', '</i>'),
+		STRONG: ('<b>', '</b>'),
+		MARK: ('<u>', '</u>'),
+		STRIKE: ('<s>', '</s>'),
+		VERBATIM: ('<tt>', '</tt>'),
+		TAG: ('<span class="zim-tag">', '</span>'),
+		SUBSCRIPT: ('<sub>', '</sub>'),
+		SUPERSCRIPT: ('<sup>', '</sup>'),
 
 	}
 
@@ -76,7 +75,7 @@ class Dumper(DumperClass):
 
 	def text(self, text):
 		if self.context[-1].tag == FORMATTEDTEXT \
-		and	text.isspace():
+		and text.isspace():
 			# Reduce top level empty lines
 			if self.template_options['empty_lines'] == 'remove':
 				self.context[-1].text.append('\n')
@@ -146,16 +145,16 @@ class Dumper(DumperClass):
 		myattrib = ''
 		if 'start' in attrib:
 			start = attrib['start']
-			if start in string.lowercase:
+			if start in 'abcdefghijklmnopqrstuvwxyz':
 				type = 'a'
-				start = string.lowercase.index(start) + 1
-			elif start in string.uppercase:
+				start = 'abcdefghijklmnopqrstuvwxyz'.index(start) + 1
+			elif start in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
 				type = 'A'
-				start = string.uppercase.index(start) + 1
+				start = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.index(start) + 1
 			else:
 				type = '1'
 			return self.dump_block(tag, attrib, strings,
-				_extra='type="%s" start="%s"' % (type, start) )
+				_extra='type="%s" start="%s"' % (type, start))
 		else:
 			return self.dump_block(tag, attrib, strings)
 
@@ -184,13 +183,13 @@ class Dumper(DumperClass):
 		href = self.linker.link(attrib['href'])
 		type = link_type(attrib['href'])
 		if strings:
-			text = u''.join(strings)
+			text = ''.join(strings)
 		else:
 			text = attrib['href']
 		title = text.replace('"', '&quot;')
 		return [
 			'<a href="%s" title="%s" class="%s">%s</a>'
-				% (href, title, type, text) ]
+				% (href, title, type, text)]
 
 	def dump_img(self, tag, attrib, strings=None):
 		src = self.linker.img(attrib['src'])
@@ -206,17 +205,61 @@ class Dumper(DumperClass):
 		else:
 			return ['<img src="%s"%s>' % (src, opt)]
 
-	def dump_object(self, *arg, **kwarg):
-		strings = DumperClass.dump_object(self, *arg, **kwarg)
-		strings.insert(0, '<div class="zim-object">\n')
-		strings.append('</div>\n')
+	def dump_object(self, tag, attrib, strings=[]):
+		strings = DumperClass.dump_object(self, tag, attrib, strings)
+		if not attrib['type'].startswith('image+'):
+			strings.insert(0, '<div class="zim-object">\n')
+			strings.append('</div>\n')
 		return strings
 
 	def dump_object_fallback(self, tag, attrib, strings=None):
 		# Fallback to verbatim paragraph
-		strings.insert(0, '<pre>\n')
-		strings.append('</pre>\n')
+		return ['<pre>\n'] + list(map(html_encode, strings)) + ['</pre>\n']
+
+	def dump_table(self, tag, attrib, strings):
+		aligns = attrib['aligns'].split(',')
+		tdcount = 0
+
+		def align(pos):
+			if pos == 'left' or pos == 'right' or pos == 'center':
+				return ' align="' + pos + '"'
+			return ''
+
+		for i, string in enumerate(strings):
+			if '<tr' in string:
+				tdcount = 0
+			elif '<th' in string:
+				strings[i] = string.replace('<th', '<th' + align(aligns[tdcount]))
+				tdcount += 1
+			elif '<td' in string:
+				strings[i] = string.replace('<td', '<td' + align(aligns[tdcount]))
+				tdcount += 1
+
+		strings.insert(0, '<table>\n')
+		strings.append('</table>\n')
 		return strings
 
-		# TODO put content in attrib, use text for caption (with full recursion)
-		# See img
+	def dump_thead(self, tag, attrib, strings):
+		strings.insert(0, '<thead><tr>\n')
+		strings.append('</tr></thead>\n')
+		return strings
+
+	def dump_th(self, tag, attrib, strings):
+		strings.insert(0, '  <th>')
+		strings.append('</th>\n')
+		return strings
+
+	def dump_trow(self, tag, attrib, strings):
+		strings.insert(0, '<tr>\n')
+		strings.append('</tr>\n')
+		return strings
+
+	def dump_td(self, tag, attrib, strings):
+		if strings == [" "]:
+			strings = ["&nbsp;"]
+		strings.insert(0, '  <td>')
+		strings.append('</td>\n')
+		return strings
+
+	def dump_line(self, tag, attrib, strings=None):
+		return '\n<hr>\n'
