@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2013-2016 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -8,6 +7,10 @@ specific commandline commands and an singleton application object that
 takes core of the process life cycle.
 '''
 
+# TODO:
+# - implement weakvalue dict to ensure uniqueness of notebook objects
+
+
 import os
 import sys
 import logging
@@ -27,6 +30,10 @@ from zim.utils import get_module, lookup_subclass
 from zim.errors import Error
 from zim.notebook import Notebook, Path, \
 	get_notebook_list, resolve_notebook, build_notebook
+from zim.formats import get_format
+
+from zim.config import ConfigManager
+from zim.plugins import PluginManager
 
 from .command import Command, GtkCommand, UsageError, GetoptError
 from .ipc import dispatch as _ipc_dispatch
@@ -92,17 +99,17 @@ Try 'zim --manual' for more help.
 '''
 
 	def run(self):
-		print self.usagehelp
-		print self.optionhelp  # TODO - generate from commands
+		print(self.usagehelp)
+		print(self.optionhelp)  # TODO - generate from commands
 
 
 class VersionCommand(Command):
 	'''Class implementing the C{--version} command'''
 
 	def run(self):
-		print 'zim %s\n' % zim.__version__
-		print zim.__copyright__, '\n'
-		print zim.__license__
+		print('zim %s\n' % zim.__version__)
+		print(zim.__copyright__, '\n')
+		print(zim.__license__)
 
 
 class NotebookLookupError(Error):
@@ -119,12 +126,14 @@ class NotebookCommand(Command):
 		'''Helper to get a default notebook'''
 		notebooks = get_notebook_list()
 		if notebooks.default:
-			return notebooks.default.uri
+			uri = notebooks.default.uri
 		elif len(notebooks) == 1:
-			return notebooks[0].uri
+			uri = notebooks[0].uri
 		else:
 			return None
 
+		return resolve_notebook(uri, pwd=self.pwd) # None if not found
+
 	def get_notebook_argument(self):
 		'''Get the notebook and page arguments for this command
 		@returns: a 2-tuple of an L{NotebookInfo} object and an
@@ -138,18 +147,15 @@ class NotebookCommand(Command):
 		notebook = args[0]
 
 		if notebook is None:
-			notebook = self.get_default_or_only_notebook()
-			if notebook:
-				logger.info('Using default notebook: %s', notebook)
-			elif self.arguments[0] == 'NOTEBOOK': # not optional
-				raise NotebookLookupError, _('Please specify a notebook')
+			if self.arguments[0] == 'NOTEBOOK': # not optional
+				raise NotebookLookupError(_('Please specify a notebook'))
 					# T: Error when looking up a notebook
 			else:
 				return None, None
 
 		notebookinfo = resolve_notebook(notebook, pwd=self.pwd)
 		if not notebookinfo:
-			raise NotebookLookupError, _('Could not find notebook: %s') % notebook
+			raise NotebookLookupError(_('Could not find notebook: %s') % notebook)
 				# T: error message
 
 		if len(self.arguments) > 1 \
@@ -176,12 +182,13 @@ class NotebookCommand(Command):
 		# mounting is attempted by zim.notebook.build_notebook()
 		notebookinfo, page = self.get_notebook_argument() 	# can raise NotebookLookupError
 		if not notebookinfo:
-			raise NotebookLookupError, _('Please specify a notebook')
+			raise NotebookLookupError(_('Please specify a notebook'))
 		notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
 
 		if ensure_uptodate and not notebook.index.is_uptodate:
 			for info in notebook.index.update_iter():
-				logger.info('Indexing %s', info)
+				#logger.info('Indexing %s', info)
+				pass # TODO meaningful info for above message
 
 		return notebook, page or uripage
 
@@ -197,56 +204,145 @@ class GuiCommand(NotebookCommand, GtkCommand):
 		('standalone', '', 'start a single instance, no background process'),
 	)
 
-	def get_notebook_argument(self):
-		def prompt():
-			import zim.gui.notebookdialog
-			notebookinfo = zim.gui.notebookdialog.prompt_notebook()
-			return notebookinfo, None
+	def build_notebook(self, ensure_uptodate=False):
+		# Bit more complicated here due to options to use default and
+		# allow using notebookdialog to prompt
 
+		# Explicit page argument has priority over implicit from uri
+		# mounting is attempted by zim.notebook.build_notebook()
+
+		from zim.notebook import FileNotFoundError
+
+		def prompt_notebook_list():
+			import zim.gui.notebookdialog
+			return zim.gui.notebookdialog.prompt_notebook()
+				# Can return None if dialog is cancelled
+
+		used_default = False
+		page = None
 		if self.opts.get('list'):
-			return prompt()
+			notebookinfo = prompt_notebook_list()
 		else:
-			notebookinfo, page = NotebookCommand.get_notebook_argument(self)
-			if not notebookinfo:
-				return prompt()
+			notebookinfo, page = self.get_notebook_argument()
+
+			if notebookinfo is None:
+				notebookinfo = self.get_default_or_only_notebook()
+				used_default = notebookinfo is not None
+
+				if notebookinfo is None:
+					notebookinfo = prompt_notebook_list()
+
+		if notebookinfo is None:
+			return None, None # Cancelled prompt
+
+		try:
+			notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
+		except FileNotFoundError:
+			if used_default:
+				# Default notebook went missing? Fallback to dialog to allow changing it
+				notebookinfo = prompt_notebook_list()
+				if notebookinfo is None:
+					return None, None # Cancelled prompt
+				notebook, uripage = build_notebook(notebookinfo) # can raise FileNotFound
 			else:
-				return notebookinfo, page
+				raise
+
+		if ensure_uptodate and not notebook.index.is_uptodate:
+			for info in notebook.index.update_iter():
+				#logger.info('Indexing %s', info)
+				pass # TODO meaningful info for above message
+
+		return notebook, page or uripage
 
 	def run(self):
-		notebook, page = self.build_notebook(ensure_uptodate=False)
-		if not notebook:
-			return # Cancelled notebook dialog
+		from gi.repository import Gtk
 
-		import gtk
-		import zim.gui
+		from zim.gui.mainwindow import MainWindow
 
-		gui = None
-		for window in gtk.window_list_toplevels():
-			if isinstance(window, zim.gui.MainWindow) \
-			and window.ui.notebook.uri == notebook.uri:
-				gui = window.ui # XXX
-				break
+		windows = [
+			w for w in Gtk.Window.list_toplevels()
+				if isinstance(w, MainWindow)
+		]
 
-		if gui:
-			gui.present(
-				page=page,
-				**self.get_options('geometry', 'fullscreen'))
+		notebook, page = self.build_notebook()
+		if notebook is None:
+			logger.debug('NotebookDialog cancelled - exit')
+			return
+
+		for window in windows:
+			if window.notebook.uri == notebook.uri:
+				self._present_window(window, page)
+				return window
 		else:
-			gui = zim.gui.GtkInterface(
-				notebook=notebook,
-				page=page,
-				**self.get_options('geometry', 'fullscreen')
-			)
-			gui.run()
+			return self._run_new_window(notebook, page)
 
-		return gui._mainwindow # XXX
+	def _present_window(self, window, page):
+		window.present()
+
+		if page:
+			window.open_page(page)
+
+		geometry = self.opts.get('geometry', None)
+		if geometry is not None:
+			window.parse_geometry(geometry)
+
+		if self.opts.get('fullscreen', False):
+			window.toggle_fullscreen(True)
+
+	def _run_new_window(self, notebook, page):
+		from gi.repository import GObject
+
+		from zim.gui.mainwindow import MainWindow
+
+		pluginmanager = PluginManager()
+
+		preferences = ConfigManager.preferences['General']
+		preferences.setdefault('plugins_list_version', 'none')
+		if preferences['plugins_list_version'] != '0.70':
+			if not preferences['plugins']:
+				pluginmanager.load_plugins_from_preferences(
+					[ # Default plugins
+						'pageindex', 'pathbar',
+						'insertsymbol', 'printtobrowser',
+						'versioncontrol',
+					]
+				)
+			else:
+				# Upgrade version <0.70 where these were core functions
+				pluginmanager.load_plugins_from_preferences(['pageindex', 'pathbar'])
+
+			if 'calendar' in pluginmanager.failed:
+				ConfigManager.preferences['JournalPlugin'] = \
+						ConfigManager.preferences['CalendarPlugin']
+				pluginmanager.load_plugins_from_preferences(['journal'])
+
+			preferences['plugins_list_version'] = '0.70'
+
+		window = MainWindow(
+			notebook,
+			page=page,
+			**self.get_options('geometry', 'fullscreen')
+		)
+		window.present()
+
+		if not window.notebook.index.is_uptodate:
+			window._uiactions.reload_index(update_only=True) # XXX
+		else:
+			# Start a lightweight background check of the index
+			# put a small delay to ensure window is shown before we start
+			def start_background_check():
+				notebook.index.start_background_check(notebook)
+				return False # only run once
+			GObject.timeout_add(500, start_background_check)
+
+		return window
 
 
 class ManualCommand(GuiCommand):
 	'''Like L{GuiCommand} but always opens the manual'''
 
 	arguments = ('[PAGE]',)
-	options = filter(lambda t: t[0] != 'list', GuiCommand.options)
+	options = tuple(t for t in GuiCommand.options if t[0] != 'list')
 		# exclude --list
 
 	def run(self):
@@ -296,6 +392,9 @@ class ServerGuiCommand(NotebookCommand, GtkCommand):
 		import zim.gui.server
 		self.opts['port'] = int(self.opts.get('port', 8080))
 		notebookinfo, page = self.get_notebook_argument()
+		if notebookinfo is None:
+			# Prefer default to be selected in drop down, user can still change
+			notebookinfo = self.get_default_or_only_notebook()
 
 		window = zim.gui.server.ServerWindow(
 			notebookinfo,
@@ -331,7 +430,7 @@ class ExportCommand(NotebookCommand):
 
 		format = self.opts.get('format', 'html')
 		if not 'output' in self.opts:
-			raise UsageError, _('Output location needed for export') # T: error in export command
+			raise UsageError(_('Output location needed for export')) # T: error in export command
 		output = Dir(self.opts['output'])
 		if not output.isdir():
 			output = File(self.opts.get('output'))
@@ -340,42 +439,46 @@ class ExportCommand(NotebookCommand):
 		if output.exists() and not self.opts.get('overwrite'):
 			if output.isdir():
 				if len(output.list()) > 0:
-					raise Error, _('Output folder exists and not empty, specify "--overwrite" to force export')  # T: error message for export
+					raise Error(_('Output folder exists and not empty, specify "--overwrite" to force export'))  # T: error message for export
 				else:
 					pass
 			else:
-				raise Error, _('Output file exists, specify "--overwrite" to force export')  # T: error message for export
+				raise Error(_('Output file exists, specify "--overwrite" to force export'))  # T: error message for export
 
 		if format == 'mhtml':
 			self.ignore_options('index-page')
 			if output.isdir():
-				raise UsageError, _('Need output file to export MHTML') # T: error message for export
+				raise UsageError(_('Need output file to export MHTML')) # T: error message for export
 
 			exporter = build_mhtml_file_exporter(
 				output, template,
 				document_root_url=self.opts.get('root-url'),
 			)
+		elif self.opts.get('singlefile'):
+			self.ignore_options('index-page')
+			if output.exists() and output.isdir():
+				ext = get_format(format).info['extension']
+				output = output.file(page.basename) + '.' + ext
+
+			exporter = build_single_file_exporter(
+				output, format, template, namespace=page,
+				document_root_url=self.opts.get('root-url'),
+			)
 		elif page:
 			self.ignore_options('index-page')
 			if output.exists() and output.isdir():
-				ext = 'html'
+				ext = get_format(format).info['extension']
 				output = output.file(page.basename) + '.' + ext
 
-			if self.opts.get('singlefile'):
-				exporter = build_single_file_exporter(
-					output, format, template, namespace=page,
-					document_root_url=self.opts.get('root-url'),
-				)
-			else:
-				exporter = build_page_exporter(
-					output, format, template, page,
-					document_root_url=self.opts.get('root-url'),
-				)
+			exporter = build_page_exporter(
+				output, format, template, page,
+				document_root_url=self.opts.get('root-url'),
+			)
 		else:
 			if not output.exists():
 				output = Dir(output.path)
 			elif not output.isdir():
-				raise UsageError, _('Need output folder to export full notebook') # T: error message for export
+				raise UsageError(_('Need output folder to export full notebook')) # T: error message for export
 
 			exporter = build_notebook_exporter(
 				output, format, template,
@@ -387,16 +490,9 @@ class ExportCommand(NotebookCommand):
 
 	def run(self):
 		from zim.export.selections import AllPages, SinglePage, SubPages
-		from zim.plugins import PluginManager
-		from zim.config import ConfigManager
 
 		notebook, page = self.build_notebook()
-
-        # load plugins, needed so the the proper export functions would work from CLI
-		config = ConfigManager(profile=notebook.profile)
-		plugins = PluginManager(config)
-		plugins.extend(notebook.index)
-		plugins.extend(notebook)
+		notebook.index.check_and_update()
 
 		if page and self.opts.get('recursive'):
 			selection = SubPages(notebook, page)
@@ -425,12 +521,12 @@ class SearchCommand(NotebookCommand):
 			logger.info('Searching for: %s', query)
 			query = Query(query)
 		else:
-			raise ValueError, 'Empty query'
+			raise ValueError('Empty query')
 
 		selection = SearchSelection(notebook)
 		selection.search(query)
 		for path in sorted(selection, key=lambda p: p.name):
-			print path.name
+			print(path.name)
 
 
 class IndexCommand(NotebookCommand):
@@ -442,11 +538,12 @@ class IndexCommand(NotebookCommand):
 		notebook, p = self.build_notebook(ensure_uptodate=False)
 		notebook.index.flush()
 		for info in notebook.index.update_iter():
-			logger.info('Indexing %s', info)
+			#logger.info('Indexing %s', info)
+			pass # TODO meaningful info for above message
 
 
 commands = {
-	'help':  HelpCommand,
+	'help': HelpCommand,
 	'version': VersionCommand,
 	'gui': GuiCommand,
 	'manual': ManualCommand,
@@ -458,7 +555,7 @@ commands = {
 }
 
 
-def build_command(args):
+def build_command(args, pwd=None):
 	'''Parse all commandline options
 	@returns: a L{Command} object
 	@raises UsageError: if args is not correct
@@ -470,7 +567,7 @@ def build_command(args):
 		try:
 			cmd = args.pop(0)
 		except IndexError:
-			raise UsageError, 'Missing plugin name'
+			raise UsageError('Missing plugin name')
 
 		try:
 			mod = get_module('zim.plugins.' + cmd)
@@ -480,7 +577,7 @@ def build_command(args):
 				logger.exception('Error while loading: zim.plugins.%s.Command', cmd)
 				# Can't use following because log level not yet set:
 				# logger.debug('Error while loading: zim.plugins.%s.Command', cmd, exc_info=sys.exc_info())
-			raise UsageError, 'Could not load commandline command for plugin "%s"' % cmd
+			raise UsageError('Could not load commandline command for plugin "%s"' % cmd)
 	else:
 		if args and args[0].startswith('--') and args[0][2:] in commands:
 			cmd = args.pop(0)[2:]
@@ -498,7 +595,7 @@ def build_command(args):
 
 		klass = commands[cmd]
 
-	obj = klass(cmd)
+	obj = klass(cmd, pwd=pwd)
 	obj.parse_options(*args)
 	return obj
 
@@ -521,6 +618,35 @@ class ZimApplication(object):
 		self._standalone = False
 		self._windows = set()
 
+	@property
+	def toplevels(self):
+		return iter(self._windows)
+
+	@property
+	def notebooks(self):
+		return frozenset(
+			w.notebook for w in self.toplevels
+				if hasattr(w, 'notebook')
+		)
+
+	def get_mainwindow(self, notebook, _class=None):
+		'''Returns an existing L{MainWindow} for C{notebook} or C{None}'''
+		from zim.gui.mainwindow import MainWindow
+		_class = _class or MainWindow # test seam
+		for w in self.toplevels:
+			if isinstance(w, _class) and w.notebook.uri == notebook.uri:
+				return w
+		else:
+			return None
+
+	def present(self, notebook, page=None):
+		'''Present notebook and page in a mainwindow, may not return for
+		standalone processes.
+		'''
+		uri = notebook if isinstance(notebook, str) else notebook.uri
+		pagename = page if isinstance(page, str) else page.name
+		self.run('--gui', uri, pagename)
+
 	def add_window(self, window):
 		if not window in self._windows:
 			logger.debug('Add window: %s', window.__class__.__name__)
@@ -539,18 +665,22 @@ class ZimApplication(object):
 	def _on_destroy_window(self, window):
 		self.remove_window(window)
 		if not self._windows:
-			import gtk
+			from gi.repository import Gtk
 
 			logger.debug('Last toplevel destroyed, quit')
-			if gtk.main_level() > 0:
-				gtk.main_quit()
+			if Gtk.main_level() > 0:
+				Gtk.main_quit()
 
-	def run(self, *args):
+	def run(self, *args, **kwargs):
 		'''Run a commandline command, either in this process, an
 		existing process, or a new process.
 		@param args: commandline arguments
+		@param kwargs: optional arguments for L{build_command}
 		'''
-		cmd = build_command(args)
+		PluginManager().load_plugins_from_preferences(
+			ConfigManager.preferences['General']['plugins']
+		)
+		cmd = build_command(args, **kwargs)
 		self._run_cmd(cmd, args) # test seam
 
 	def _run_cmd(self, cmd, args):
@@ -558,18 +688,26 @@ class ZimApplication(object):
 
 		if self._running:
 			# This is not the first command that we process
-			if self._standalone or cmd.opts.get('standalone'):
-				self._spawn_standalone(args)
-			elif isinstance(cmd, GtkCommand):
-				w = cmd.run()
-				if w is not None:
-					self.add_window(w)
+			if isinstance(cmd, GtkCommand):
+				if self._standalone or cmd.standalone_process:
+					self._spawn_standalone(args)
+				else:
+					w = cmd.run()
+					if w is not None:
+						self.add_window(w)
+						w.present()
 			else:
 				cmd.run()
 		else:
-			self._standalone = cmd.opts.get('standalone')
+			# Although a-typical, this path could be re-entrant if a
+			# run_local() dispatches another command - therefore we set
+			# standalone before calling run_local()
 			if isinstance(cmd, GtkCommand):
-				if not self._standalone and self._try_dispatch(args):
+				self._standalone = self._standalone or cmd.standalone_process
+				if cmd.run_local():
+					return
+
+				if not self._standalone and self._try_dispatch(args, cmd.pwd):
 					pass # We are done
 				else:
 					self._running = True
@@ -580,8 +718,25 @@ class ZimApplication(object):
 	def _run_main_loop(self, cmd):
 		# Run for the 1st gtk command in a primary process,
 		# but can still be standalone process
-		import gtk, gobject
-		gobject.threads_init()
+		from gi.repository import Gtk
+		from gi.repository import GObject
+
+		#######################################################################
+		# WARNING: commented out "GObject.threads_init()" because it leads to
+		# various segfaults on linux. See github issue #7
+		# However without this init, gobject does not properly release the
+		# python GIL during C calls, so threads may block while main loop is
+		# waiting. Thus threads become very slow and unpredictable unless we
+		# actively monitor them from the mainloop, causing python to run
+		# frequently. So be very carefull relying on threads.
+		# Re-evaluate when we are above PyGObject 3.10.2 - threading should
+		# wotk bettter there even without this statement. (But even then,
+		# no Gtk calls from threads, just "GObject.idle_add()". )
+		# Kept for windows, because we need thread to run ipc listener, and no
+		# crashes observed there.
+		if os.name == 'nt':
+			GObject.threads_init()
+		#######################################################################
 
 		from zim.gui.widgets import gtk_window_set_default_icon
 		gtk_window_set_default_icon()
@@ -594,20 +749,20 @@ class ZimApplication(object):
 		else:
 			logger.debug('Starting primary process')
 			self._daemonize()
-			_ipc_start_listening(self.run)
+			if not _ipc_start_listening(self._handle_incoming):
+				logger.warn('Failure to setup socket, falling back to "--standalone" mode')
+				self._standalone = True
 
 		w = cmd.run()
 		if w is not None:
 			self.add_window(w)
 
 		while self._windows:
-			gtk.main()
+			Gtk.main()
 
 			for toplevel in list(self._windows):
 				try:
 					toplevel.destroy()
-					while gtk.events_pending():
-						gtk.main_iteration(block=False)
 				except:
 					logger.exception('Exception while destroying window')
 					self.remove_window(toplevel) # force removal
@@ -643,16 +798,15 @@ class ZimApplication(object):
 
 			logger.debug('Python version is %s', str(sys.version_info))
 			logger.debug('Platform is %s', os.name)
-			logger.debug(zim.get_zim_revision())
 			zim.config.log_basedirs()
 
 	def _setup_signal_handling(self):
 		def handle_sigterm(signal, frame):
-			import gtk
+			from gi.repository import Gtk
 
 			logger.info('Got SIGTERM, quit')
-			if gtk.main_level() > 0:
-				gtk.main_quit()
+			if Gtk.main_level() > 0:
+				Gtk.main_quit()
 
 		signal.signal(signal.SIGTERM, handle_sigterm)
 
@@ -673,10 +827,10 @@ class ZimApplication(object):
 
 		Application([ZIM_EXECUTABLE] + args).spawn()
 
-	def _try_dispatch(self, args):
+	def _try_dispatch(self, args, pwd):
 		try:
-			_ipc_dispatch(*args)
-		except AssertionError, err:
+			_ipc_dispatch(pwd, *args)
+		except AssertionError as err:
 			logger.debug('Got error in dispatch: %s', str(err))
 			return False
 		except Exception:
@@ -686,6 +840,9 @@ class ZimApplication(object):
 			logger.debug('Dispatched command %r', args)
 			return True
 
+	def _handle_incoming(self, pwd, *args):
+		self.run(*args, pwd=pwd)
+
 	def _daemonize(self):
 		# Decouple from parent environment
 		# and redirect standard file descriptors
@@ -709,10 +866,16 @@ class ZimApplication(object):
 			dir = zim.fs.get_tmpdir()
 			err_stream = open(os.path.join(dir.path, "zim.log"), "w")
 
+			# Try to flush standards out and error, if there
+			for pipe in (sys.stdout, sys.stderr):
+				if pipe is not None:
+					try:
+						pipe.flush()
+					except OSError:
+						pass
+
 			# First try to dup handles for anyone who still has a reference
 			# if that fails, just set them (maybe not real files in the first place)
-			sys.stdout.flush()
-			sys.stderr.flush()
 			try:
 				os.dup2(err_stream.fileno(), sys.stdout.fileno())
 				os.dup2(err_stream.fileno(), sys.stderr.fileno())
@@ -741,6 +904,19 @@ def main(*argv):
 	'''Run full zim application
 	@returns: exit code (if error handled, else just raises)
 	'''
+
+	import zim.config
+
+	# Check if we can find our own data files
+	_file = zim.config.data_file('zim.png')
+	if not (_file and _file.exists()): #pragma: no cover
+		raise AssertionError(
+			'ERROR: Could not find data files in path: \n'
+			'%s\n'
+			'Try setting XDG_DATA_DIRS'
+				% list(map(str, zim.config.data_dirs()))
+		)
+
 	try:
 		ZIM_APPLICATION.run(*argv[1:])
 	except KeyboardInterrupt:
