@@ -1,10 +1,9 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # Copyright 2010,2011 John Drinkwater <john@nextraweb.com>
 # Copyright 2012 Damien Accorsi <damien.accorsi@free.fr>
 
-from __future__ import with_statement
+
 
 import os
 import logging
@@ -18,9 +17,6 @@ logger = logging.getLogger('zim.vcs.git')
 
 class GITApplicationBackend(VCSApplicationBase):
 
-	def __init__(self, root):
-		VCSApplicationBase.__init__(self, root)
-
 	@classmethod
 	def build_bin_application_instance(cls):
 		return Application(('git',))
@@ -38,7 +34,7 @@ class GITApplicationBackend(VCSApplicationBase):
 		-r revision
 		-r rev1..rev2
 		"""
-		if is_for_diff==True:
+		if is_for_diff:
 			if len(versions) == 2:
 				versions.reverse()
 				return ['..'.join(versions)]
@@ -50,7 +46,7 @@ class GITApplicationBackend(VCSApplicationBase):
 			if isinstance(versions, (tuple, list)):
 				assert 1 <= len(versions) <= 2
 				if len(versions) == 2:
-					return map(str, versions)
+					return list(map(str, versions))
 				else:
 					versions = versions[0]
 
@@ -60,6 +56,12 @@ class GITApplicationBackend(VCSApplicationBase):
 			else:
 				return []
 
+	def path_arg(self, path):
+		# make sure "/" is used as path seperator as git under msys fails otherwise
+		if isinstance(path, str):
+			return path.replace('\\', '/')
+		else:
+			return path.relpath(self.root).replace('\\', '/')
 
 	########
 	#
@@ -70,12 +72,12 @@ class GITApplicationBackend(VCSApplicationBase):
 		Runs: git add {{PATH}}
 		"""
 		if path is None:
-			return self.run(['add', '.'])
+			return self.run(['add', self.notebook_dir])
 		else:
-			return self.run(['add', path])
+			return self.run(['add', self.path_arg(path)])
 
 
-	def annotate(self, file, version):
+	def annotate(self, file, version=None):
 		"""FIXME Document
 		return
 		0: line1
@@ -83,30 +85,30 @@ class GITApplicationBackend(VCSApplicationBase):
 		...
 		"""
 		revision_args = self.build_revision_arguments(version)
-		return self.pipe(['blame', '-s', file] + revision_args)
+		return self.pipe(['blame', '-s', self.path_arg(file)] + revision_args)
 
 	def cat(self, path, version):
 		"""
 		Runs: git cat {{PATH}} {{REV_ARGS}}
 		"""
 		revision_args = self.build_revision_arguments(version)
-		return self.pipe(['show', ''.join( [ ''.join(revision_args), ':', path.relpath(self.root) ] )])
+		return self.pipe(['show', ''.join([''.join(revision_args), ':', self.path_arg(path)])])
 
 	def commit(self, path, msg):
 		"""
-		Runs: git commit -a -m {{MSG}} {{PATH}}
+		Runs: git commit -m {{MSG}} {{PATH}}
 		"""
 		if self.is_modified():
-			params = ['commit', '-a']
-			if msg!='' and msg!=None:
+			params = ['commit']
+			if msg != '' and msg is not None:
 				params.append('-m')
 				params.append(msg)
-			if path!='' and path!=None:
+			if path != '' and path is not None:
 				params.append('--')
-				params.append(path)
+				params.append(self.path_arg(path))
 			return self.run(params)
 
-	def diff(self, versions, path=None):
+	def diff(self, versions=None, file=None):
 		"""
 		Runs:
 			git diff --no-ext-diff {{REVISION_ARGS}}
@@ -115,17 +117,17 @@ class GITApplicationBackend(VCSApplicationBase):
 		"""
 		revision_args = self.build_revision_arguments(versions)
 		revision_args = self.build_revision_arguments(revision_args, is_for_diff=True)
-		if path==None:
+		if file is None:
 			return self.pipe(['diff', '--no-ext-diff'] + revision_args)
 		else:
-			return self.pipe(['diff', '--no-ext-diff'] + revision_args + ['--', path])
+			return self.pipe(['diff', '--no-ext-diff'] + revision_args + ['--', self.path_arg(file)])
 
 	def ignore(self, file_to_ignore_regexp):
 		"""
 		Build a .gitignore file including the file_to_ignore_content
 		"""
 		#TODO: append the rule instead of overwrite the full content
-		self.root.file( '.gitignore' ).write( file_to_ignore_regexp )
+		self.root.file('.gitignore').write(file_to_ignore_regexp)
 
 	def init_repo(self):
 		self.init()
@@ -133,7 +135,7 @@ class GITApplicationBackend(VCSApplicationBase):
 		self.add('.') # add all existing files
 
 	def repo_exists(self):
-		return self.root.subdir('.git').exists()
+		return self.root.folder('.git').exists() or self.root.file('.git').exists()
 
 	def init(self):
 		"""
@@ -146,7 +148,8 @@ class GITApplicationBackend(VCSApplicationBase):
 		@returns: True if the repo is not up-to-date, or False
 		"""
 		# If status return an empty answer, this means the local repo is up-to-date
-		return not (''.join( self.status() ).find( 'nothing to commit' ) > -1)
+		status = ''.join(self.pipe(['status', '--porcelain']))
+		return bool(status.strip())
 
 	def log(self, path=None):
 		"""
@@ -156,7 +159,7 @@ class GITApplicationBackend(VCSApplicationBase):
 			git log --date=iso
 		"""
 		if path:
-			return self.pipe(['log', '--date=iso', '--follow', path])
+			return self.pipe(['log', '--date=iso', '--follow', self.path_arg(path)])
 		else:
 			return self.pipe(['log', '--date=iso'])
 
@@ -188,7 +191,7 @@ class GITApplicationBackend(VCSApplicationBase):
 			elif line.startswith('Date: '):
 				date = line[7:].strip()
 				seenmsg = True
-				msg = u''
+				msg = ''
 			elif seenmsg and line.startswith(' '):
 				msg += line[4:]
 
@@ -200,18 +203,12 @@ class GITApplicationBackend(VCSApplicationBase):
 
 
 	def move(self, oldpath, newpath):
-		"""
-		Runs: git mv --after {{OLDPATH}} {{NEWPATH}}
-		"""
-		return self.run(['mv', '--after', oldpath, newpath])
+		return self.add(newpath) # move already happened
 
 	def remove(self, path):
-		"""
-		Runs: git rm {{PATH}}
-		"""
-		return self.run(['rm', path])
+		pass # delete already happened
 
-	def revert(self, path, version):
+	def revert(self, path=None, version=None):
 		"""
 		Runs:
 			hg revert {{PATH}} {{REV_ARGS}}
@@ -225,16 +222,19 @@ class GITApplicationBackend(VCSApplicationBase):
 		"""
 		revision_params = self.build_revision_arguments(version)
 		if path:
-			self.run(['checkout'] + revision_params + ['--', path])
+			self.run(['checkout'] + revision_params + ['--', self.path_arg(path)])
 		else:
 			self.run(['reset', '--hard', 'HEAD'])
 
 	def stage(self):
-		self.run(['add', '-u'])
-		self.run(['add', '-A'])
+		self.add()
 
-	def status(self):
+	def status(self, porcelain=False):
 		"""
 		Runs: git status
+		@param porcelain: see --porcelain in git documentation, used for testing
 		"""
-		return self.pipe(['status'])
+		if porcelain:
+			return self.pipe(['status', '--porcelain'])
+		else:
+			return self.pipe(['status'])
