@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2012-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2012-2015 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 '''Module with assorted useful classes and functions used in the zim code'''
 
+import functools
+
 
 class classproperty(object):
 	'''Like C{property()} but for klass properties
@@ -18,6 +19,7 @@ class classproperty(object):
 		return self.func(owner)
 
 
+
 ## Functions for dynamic loading of modules and klasses
 import inspect
 
@@ -52,7 +54,7 @@ def lookup_subclass(module, klass):
 	'''
 	subclasses = lookup_subclasses(module, klass)
 	if len(subclasses) > 1:
-		raise AssertionError, 'BUG: Multiple subclasses found of type: %s' % klass
+		raise AssertionError('BUG: Multiple subclasses found of type: %s' % klass)
 	elif subclasses:
 		return subclasses[0]
 	else:
@@ -128,9 +130,6 @@ def natural_sort_key(string, numeric_padding=5):
 	templ = '%0' + str(numeric_padding) + 'i'
 	string.strip()
 	string = _num_re.sub(lambda m: templ % int(m.group()), string)
-	if isinstance(string, unicode):
-		string = unicodedata.normalize('NFKC', string)
-		# may be done by strxfrm as well, but want to be sure
 	string = string.lower() # sort case insensitive
 
 	try:
@@ -138,7 +137,7 @@ def natural_sort_key(string, numeric_padding=5):
 			# 8-bit byte string - enode to hex -- in pyton3 check if byte data type is handled better by sqlite3 and others
 	except MemoryError:
 		# Known python issue :(
-		bytestring = string.encode('utf-8')
+		bytestring = string
 
 	key = ''.join(["%02x" % ord(c) for c in bytestring])
 	return key
@@ -219,7 +218,7 @@ class OrderedDict(collections.MutableMapping):
 	def __repr__(self):
 		return '<%s:\n%s\n>' % (
 			self.__class__.__name__,
-			',\n'.join('  %r: %r' % (k, v) for k, v in self.items())
+			',\n'.join('  %r: %r' % (k, v) for k, v in list(self.items()))
 		)
 
 	def __getitem__(self, k):
@@ -233,13 +232,16 @@ class OrderedDict(collections.MutableMapping):
 
 	def __delitem__(self, k):
 		del self._values[k]
-		self._keys.remove(k)
+		try:
+			self._keys.remove(k)
+		except ValueError:
+			pass
 
 	def __iter__(self):
-		return iter(self._keys)
+		return iter(k for k in self._keys if k in self._values)
 
 	def __len__(self):
-		return len(self._keys)
+		return len(self._values)
 
 
 ## Special iterator class
@@ -259,7 +261,7 @@ class MovingWindowIter(object):
 	def __init__(self, iterable):
 		self._iter = iter(iterable)
 		try:
-			first = self._iter.next()
+			first = next(self._iter)
 		except StopIteration:
 			# empty list
 			self.last = True
@@ -271,71 +273,17 @@ class MovingWindowIter(object):
 	def __iter__(self):
 		return self
 
-	def next(self):
+	def __next__(self):
 		if self.last:
 			raise StopIteration
 
 		discard, prev, current = self.items
 		try:
-			next = self._iter.next()
+			mynext = next(self._iter)
 		except StopIteration:
 			self.last = True
 			self.items = (prev, current, None)
 		else:
-			self.items = (prev, current, next)
+			self.items = (prev, current, mynext)
 
 		return self.items
-
-
-
-## Wrapper for using threads for e.g. async IO
-import threading
-import sys
-
-
-class FunctionThread(threading.Thread):
-	'''Subclass of C{threading.Thread} that runs a single function and
-	keeps the result and any exceptions raised.
-
-	@ivar done: C{True} is the function is done running
-	@ivar result: the return value of C{func}
-	@ivar error: C{True} if done and an exception was raised
-	@ivar exc_info: 3-tuple with exc_info
-	'''
-
-	def __init__(self, func, args=(), kwargs={}, lock=None):
-		'''Constructor
-		@param func: the function to run in the thread
-		@param args: arguments for C{func}
-		@param kwargs: keyword arguments for C{func}
-		@param lock: optional lock, will be acquired in main thread
-		before running and released once done in background
-		'''
-		threading.Thread.__init__(self)
-
-		self.func = func
-		self.args = args
-		self.kwargs = kwargs
-
-		self.lock = lock
-
-		self.done = False
-		self.result = None
-		self.error = False
-		self.exc_info = (None, None, None)
-
-	def start(self):
-		if self.lock:
-			self.lock.acquire()
-		threading.Thread.start(self)
-
-	def run(self):
-		try:
-			self.result = self.func(*self.args, **self.kwargs)
-		except:
-			self.error = True
-			self.exc_info = sys.exc_info()
-		finally:
-			self.done = True
-			if self.lock:
-				self.lock.release()
