@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -22,23 +21,24 @@ the standalone server.
 import sys
 import socket
 import logging
-import gobject
+from gi.repository import GObject
 
 from functools import partial
 
 from wsgiref.headers import Headers
-import urllib
+import urllib.request
+import urllib.parse
+import urllib.error
 
 from zim.errors import Error
-from zim.notebook import Notebook, Path, Page, IndexPage, PageNameError
+from zim.notebook import Notebook, Path, Page, encode_filename, PageNotFoundError
 from zim.fs import File, Dir, FileNotFoundError
-from zim.config import data_file, ConfigManager
-from zim.plugins import PluginManager
-from zim.stores import encode_filename
+from zim.config import data_file
 from zim.parsing import url_encode
 
 from zim.export.linker import ExportLinker, StubLayout
 from zim.export.template import ExportTemplateContext
+from zim.export.exporters import createIndexPage
 
 from zim.formats import get_format
 
@@ -73,7 +73,7 @@ class WWWError(Error):
 			self.msg += ' - ' + msg
 
 
-class PageNotFoundError(WWWError):
+class WebPageNotFoundError(WWWError):
 	'''Error whan a page is not found (404)'''
 
 	description = '''\
@@ -81,12 +81,12 @@ You tried to open a page that does not exist.
 '''
 
 	def __init__(self, page):
-		if not isinstance(page, basestring):
+		if not isinstance(page, str):
 			page = page.name
 		WWWError.__init__(self, 'No such page: %s' % page, status='404')
 
 
-class PathNotValidError(WWWError):
+class WebPathNotValidError(WWWError):
 	'''Error when the url points to an invalid page path'''
 
 	description = '''\
@@ -108,38 +108,31 @@ class WWWInterface(object):
 	in the standard library for python.
 	'''
 
-	def __init__(self, notebook, config=None, template='Default'):
+	def __init__(self, notebook, template='Default'):
 		'''Constructor
 		@param notebook: a L{Notebook} object
-		@param config: optional C{ConfigManager} object
 		@param template: html template for zim pages
 		'''
 		assert isinstance(notebook, Notebook)
 		self.notebook = notebook
-		self.config = config or ConfigManager(profile=notebook.profile)
 
 		self.output = None
 
 		if template is None:
 			template = 'Default'
 
-		if isinstance(template, basestring):
+		if isinstance(template, str):
 			from zim.templates import get_template
 			self.template = get_template('html', template)
 			if not self.template:
-				raise AssertionError, 'Could not find html template: %s' % template
+				raise AssertionError('Could not find html template: %s' % template)
 		else:
 			self.template = template
 
 		self.linker_factory = partial(WWWLinker, self.notebook, self.template.resources_dir)
 		self.dumper_factory = get_format('html').Dumper # XXX
 
-		self.plugins = PluginManager(self.config)
-		self.plugins.extend(notebook.index)
-		self.plugins.extend(notebook)
-		self.plugins.extend(self)
-
-		#~ self.notebook.index.update()
+		#~ self.notebook.indexer.check_and_update()
 
 	def __call__(self, environ, start_response):
 		'''Main function for handling a single request. Follows the
@@ -159,29 +152,32 @@ class WWWInterface(object):
 		headerlist = []
 		headers = Headers(headerlist)
 		path = environ.get('PATH_INFO', '/')
+		path = path.encode('iso-8859-1').decode('UTF-8')
+			# The WSGI standard mandates iso-8859-1, but we want UTF-8. See:
+			# - https://www.python.org/dev/peps/pep-3333/#unicode-issues
+			# - https://code.djangoproject.com/ticket/19468
 		try:
 			methods = ('GET', 'HEAD')
 			if not environ['REQUEST_METHOD'] in methods:
 				raise WWWError('405', headers=[('Allow', ', '.join(methods))])
 
 			# cleanup path
-			#~ print 'INPUT', path
 			path = path.replace('\\', '/') # make it windows save
 			isdir = path.endswith('/')
 			parts = [p for p in path.split('/') if p and not p == '.']
 			if [p for p in parts if p.startswith('.')]:
 				# exclude .. and all hidden files from possible paths
-				raise PathNotValidError()
+				raise WebPathNotValidError()
 			path = '/' + '/'.join(parts)
-			if isdir and not path == '/': path += '/'
-			#~ print 'PATH', path
+			if isdir and not path == '/':
+				path += '/'
 
 			if not path:
 				path = '/'
 			elif path == '/favicon.ico':
 				path = '/+resources/favicon.ico'
 			else:
-				path = urllib.unquote(path)
+				path = urllib.parse.unquote(path)
 
 			if path == '/':
 				headers.add_header('Content-Type', 'text/html', charset='utf-8')
@@ -189,18 +185,18 @@ class WWWInterface(object):
 			elif path.startswith('/+docs/'):
 				dir = self.notebook.document_root
 				if not dir:
-					raise PageNotFoundError(path)
+					raise WebPageNotFoundError(path)
 				file = dir.file(path[7:])
 				content = [file.raw()]
 					# Will raise FileNotFound when file does not exist
 				headers['Content-Type'] = file.get_mimetype()
 			elif path.startswith('/+file/'):
-				file = self.notebook.dir.file(path[7:])
+				file = self.notebook.folder.file(path[7:])
 					# TODO: need abstraction for getting file from top level dir ?
-				content = [file.raw()]
+				content = [file.read_binary()]
 					# Will raise FileNotFound when file does not exist
-				headers['Content-Type'] = file.get_mimetype()
- 			elif path.startswith('/+resources/'):
+				headers['Content-Type'] = file.mimetype()
+			elif path.startswith('/+resources/'):
 				if self.template.resources_dir:
 					file = self.template.resources_dir.file(path[12:])
 					if not file.exists():
@@ -212,8 +208,8 @@ class WWWInterface(object):
 					content = [file.raw()]
 						# Will raise FileNotFound when file does not exist
 					headers['Content-Type'] = file.get_mimetype()
-	 			else:
-					raise PageNotFoundError(path)
+				else:
+					raise WebPageNotFoundError(path)
 			else:
 				# Must be a page or a namespace (html file or directory path)
 				headers.add_header('Content-Type', 'text/html', charset='utf-8')
@@ -222,30 +218,33 @@ class WWWInterface(object):
 				elif path.endswith('/'):
 					pagename = path[:-1].replace('/', ':')
 				else:
-					raise PageNotFoundError(path)
+					raise WebPageNotFoundError(path)
 
-				path = self.notebook.resolve_path(pagename)
-				page = self.notebook.get_page(path)
-				if page.hascontent:
-					content = self.render_page(page)
-				elif page.haschildren:
-					content = self.render_index(page)
-				else:
-					raise PageNotFoundError(page)
-		except Exception, error:
+				path = self.notebook.pages.lookup_from_user_input(pagename)
+				try:
+					page = self.notebook.get_page(path)
+					if page.hascontent:
+						content = self.render_page(page)
+					elif page.haschildren:
+						content = self.render_index(page)
+					else:
+						raise WebPageNotFoundError(path)
+				except PageNotFoundError:
+					raise WebPageNotFoundError(path)
+		except Exception as error:
 			headerlist = []
 			headers = Headers(headerlist)
 			headers.add_header('Content-Type', 'text/plain', charset='utf-8')
 			if isinstance(error, (WWWError, FileNotFoundError)):
 				logger.error(error.msg)
 				if isinstance(error, FileNotFoundError):
-					error = PageNotFoundError(path)
+					error = WebPageNotFoundError(path)
 					# show url path instead of file path
 				if error.headers:
 					for key, value in error.headers:
 						headers.add_header(key, value)
 				start_response(error.status, headerlist)
-				content = unicode(error).splitlines(True)
+				content = str(error).splitlines(True)
 			# TODO also handle template errors as special here
 			else:
 				# Unexpected error - maybe a bug, do not expose output on bugs
@@ -253,16 +252,17 @@ class WWWInterface(object):
 				logger.exception('Unexpected error:')
 				start_response('500 Internal Server Error', headerlist)
 				content = ['Internal Server Error']
+
 			if environ['REQUEST_METHOD'] == 'HEAD':
 				return []
 			else:
-				return [string.encode('utf-8') for string in content]
+				return [c.encode('UTF-8') for c in content]
 		else:
 			start_response('200 OK', headerlist)
 			if environ['REQUEST_METHOD'] == 'HEAD':
 				return []
-			elif 'utf-8' in headers['Content-Type']:
-				return [string.encode('utf-8') for string in content]
+			elif content and isinstance(content[0], str):
+				return [c.encode('UTF-8') for c in content]
 			else:
 				return content
 
@@ -271,7 +271,8 @@ class WWWInterface(object):
 		@param namespace: the namespace L{Path}
 		@returns: html as a list of lines
 		'''
-		page = IndexPage(self.notebook, namespace)
+		path = namespace or Path(':')
+		page = createIndexPage(self.notebook, path, namespace)
 		return self.render_page(page)
 
 	def render_page(self, page):
@@ -289,10 +290,10 @@ class WWWInterface(object):
 			content=[page],
 			home=self.notebook.get_home_page(),
 			up=page.parent if page.parent and not page.parent.isroot else None,
-			prevpage=self.notebook.index.get_previous(page),
-			nextpage=self.notebook.index.get_next(page),
+			prevpage=self.notebook.pages.get_previous(page) if not page.isroot else None,
+			nextpage=self.notebook.pages.get_next(page) if not page.isroot else None,
 			links={'index': '/'},
-			index_generator=self.notebook.index.walk,
+			index_generator=self.notebook.pages.walk,
 			index_page=page,
 		)
 		self.template.process(lines, context)
@@ -324,9 +325,9 @@ class WWWLinker(ExportLinker):
 
 	def file_object(self, file):
 		'''Turn a L{File} object in a relative link or URI'''
-		if file.ischild(self.notebook.dir):
+		if file.ischild(self.notebook.folder):
 			# attachment
-			relpath = file.relpath(self.notebook.dir)
+			relpath = file.relpath(self.notebook.folder)
 			return url_encode('/+file/' + relpath)
 		elif self.notebook.document_root \
 		and file.ischild(self.notebook.document_root):
